@@ -59,7 +59,9 @@ typedef struct note
   enum headtype noteheadtype; /**< Holds note head type.  */
   gint x_off;  /**< Holds x offset from default position.  */
   gint y_off; /**< Holds y offset from default position.  */
-  GString *directive; /**< LilyPond text to be inserted after the note */
+  GString *prefix;  /**< LilyPond text to be inserted before the note */
+  GString *postfix; /**< LilyPond text to be inserted after the note */
+  GString *display; /**< something to display to describe the LilyPond attached to the note */
 }
 note;
 
@@ -91,7 +93,7 @@ typedef enum ornament {
   PRALL,
   PRALLPRALL,
   SEGNO,
-  SFORZATO,
+  SFORZATO,/*unused is a dynamic */
   STOPPED,
   THUMB,
   TRILL_ELEMENT,
@@ -123,6 +125,7 @@ typedef struct chord
   gint numdots; /**< Number of dots that are applied to the note */
   gint sum_mid_c_offset;
   gint numnotes;
+  gboolean chordize;/**< TRUE if this chord is to be treated as a multi-note chord even if it only has one note */
 
   gboolean is_tied;
   gboolean is_stemup;
@@ -161,6 +164,9 @@ typedef struct chord
   gboolean is_fakechord; /**< This is the actual value of the fake chord if is_fakechord */
   gpointer fakechord; /**< This is the actual value of the fake chord if is_fakechord */
   gpointer fakechord_extension; /**< This is the extension to the fake chord. It may be a 7, sus4, 9, 11, 13 etc... */
+  GString *prefix; /**< LilyPond text to be inserted before the chord */
+  GString *postfix;/**< LilyPond text to be inserted after the chord */
+  GString *display; /**< something to display to describe the LilyPond attached to the chord */
 }
 chord;
 
@@ -252,11 +258,6 @@ typedef struct barline
 }
 barline;
 
-/* Data structure for an arbitrary directive to be passed to
- * Lilypond that Denemo doesn't understand. This type is useful
- * in that the user can insert such directives into a mudela file
- * by hand and have Denemo respect them when it loads a file and
- * write them back when it saves it */
 
 
 /**
@@ -270,11 +271,20 @@ typedef struct lyric
   gboolean center_lyric; /**< Should the lyrics be centered? */
 }lyric;
 
-/**/
+
+/* Data structure for an arbitrary directive to be passed to
+ * Lilypond that Denemo doesn't understand. This type is useful
+ * in that the user can insert such directives into a LilyPond file
+ * by hand and have Denemo respect them when it loads a file and
+ * write them back when it saves it */
+
 typedef struct lilydirective
 {
-  GString *directive;
+  GString *directive;/**< the LilyPond text */
   gboolean locked;/**< If true the directive cannot be deleted easily */
+  GString *display;/**< Something for Denemo to display (to indicate what the directive is doing*/
+  gint x;/**< horizontal offset of display text */
+  gint y;/**< vertical offset of display text */
 }
 lilydirective;
 
