@@ -284,11 +284,12 @@ typedef struct DenemoKeymap
 
   GHashTable *idx_from_keystring; //hashtable linking the string representing
                                   //a keypress to the index of its command
-								  //The keystring is the output of
-								  //dnm_accelerator_name()
+                                  //The keystring is the output of
+  				  //dnm_accelerator_name()
+  GHashTable *cursors;//hashtable linking GdkEvent state to a cursor that should be used in that state
 }keymap;
 
-#define MAX_HISTORY 10
+
 /**
  * DenemoPrefs holds information on user preferences. 
  */
@@ -301,6 +302,8 @@ typedef struct DenemoPrefs
 			     wav file after rendering or not */
   gboolean immediateplayback; /**< This options sends audio directly to synth as notes 
 				are being entered */
+  gboolean overlays; /*< whether overlays or insert should be used with pitch entry */
+  gboolean continuous; /*< whether pitch entry overlays should cross barlines */
   gboolean lilyentrystyle;  
   gboolean createclones; 
   gboolean articulation_palette; /**< This switch makes the articulation pallete visable */
@@ -308,11 +311,14 @@ typedef struct DenemoPrefs
   gboolean rhythm_palette; /**< This option makes the rhythm toolbar visable */
   gboolean saveparts; /**< Automatically save parts*/
   gboolean autosave; /**< Auto save data */
-  gint autosave_timeout; 
+  gint autosave_timeout;
+  gint maxhistory;/**< how long a history of used files to retain */
   GString *browser; /**< Default browser string */
   GString *csoundcommand; /**< command used to execute csound */
   GString *csoundorcfile; /**< Path to .orc file used for csound playback */
   gboolean rtcs; /**< Real time csound */
+  GString *sequencer;  /**< path to sequencer device */
+  GString *midi_in;  /**< path to midi_in device */
   GString *pdfviewer; /**< PDF viewer */
   GString *imageviewer; /**< Image Viewer */
   GString *texteditor; /**< texteditor for editing scripts and lilypond files */
@@ -321,6 +327,8 @@ typedef struct DenemoPrefs
 
   GString *lilyversion; /**< Lilypoind Version */
   GString *temperament; /**< Preferred temperament for tuning to */
+  gboolean strictshortcuts; /**< Whether shortcuts require CaspLock & NumLock to be correct */
+  gint resolution; /**< Resolution of exported selection in dpi */
 }DenemoPrefs;
 
 /**
@@ -349,6 +357,8 @@ typedef struct LilypondHeaderFields
   /* lilypond before and after each \score block     */
   GString *lilypond_before;
   GString *lilypond_after;
+  /* preferences to go into \layout block */
+  GString *layout;
 }LilypondHeaderFields;
 
 
@@ -372,6 +382,14 @@ typedef enum
   NONE
 }InstrumentType;
 
+
+typedef enum InputSource {
+  INPUTKEYBOARD,
+  INPUTAUDIO,
+  INPUTMIDI
+} InputSource;
+
+
 /**
  * Structure to contain the list of Instruments for the score
  * configuration wizard
@@ -439,6 +457,14 @@ typedef struct DenemoLilyControl
 	
 } DenemoLilyControl;
 
+
+typedef struct DenemoScriptParam { /**< commands called by scripts use one of these to pass in a string and return a boolean */
+  GString *string;/**< input string to command */
+  gboolean status;/**< return value - TRUE = normal case execution of command/FALSE = exceptional case*/
+} DenemoScriptParam;
+
+
+
 typedef struct DenemoScoreblock {
   GString *scoreblock;/**< text of the scoreblock */
   gboolean visible;/**< Whether the scoreblock should be used by default */
@@ -501,7 +527,7 @@ typedef struct DenemoScore
   gint curmeasure_stem_directive;
 
 
-  gboolean is_grace_mode;
+
   /* Is there a figured bass present, is so this relates the bass
    * with its figures staff, if one is present */
   staff_info * has_figures;
@@ -554,7 +580,6 @@ typedef struct DenemoScore
  */
 typedef struct DenemoGUI
 {
-
   /* Fields used fairly directly for drawing */
   GtkWidget *page;
   GtkWidget *scorearea;
@@ -574,7 +599,7 @@ typedef struct DenemoGUI
   gint plugincounter;
 
   GtkWidget* articulation_palette; /**< Articulation palette window */
-  gboolean pitch_recognition;/**< Whether pitch recognition off/on */
+  InputSource input_source;/**< Where pitches are coming into Denemo (keyboard, audio, midi) */
   input_mode mode; /**< Input mode for Score */
   GtkWidget *progressbar;
 
@@ -652,7 +677,7 @@ struct DenemoRoot
   gint width;
   gint height;
   gboolean maximized;
-  keymap *commands; /**< pointer to data describing each of the Denemo commands and their keyboard shortcuts */
+  keymap *map; /**< pointer to data describing each of the Denemo commands and their keyboard shortcuts */
   gchar *last_merged_command;/**<filename of last command merged into the menu system */
   gint last_keyval, last_keystate;/**< most recent keypress which successfully invoked a command */
   GList *guis; /**< the list of DenemoGUI objects, representing pieces of music
@@ -667,11 +692,15 @@ struct DenemoRoot
   GtkWidget *notebook;/**< contains the gui.page widgets */
   GtkWidget *statusbar;
   gint status_context_id;
+  GtkWidget *input_source; /**< A label widget advising of source of external input */
+  GString *input_filters; /**< Description of any filters operating on external input */
   GtkWidget *menubar;/**< Main menubar to giving load/save play etc functionality */
   GtkWidget *ClassicModeMenu;/**< Menu to give the note editing facilities in Classic mode */
   GtkWidget *InsertModeMenu;/**< Menu to give the note editing facilities in Insert mode */
   GtkWidget *EditModeMenu;/**< Menu to give the note editing facilities in Edit mode */
   GtkWidget *ModelessMenu;/**< Menu to give the note editing facilities when used without modes */
+  gboolean QuickShortcutEdits;/**< TRUE if pressing a key while hovering over a menu item sets a shortcut */
+
   struct RhythmPattern *singleton_rhythms[256]; /**< rhythm patterns for the EntryToolbar */
   gboolean ScriptRecording;/**< TRUE when menuitems presses are being recorded as scheme script*/
   GtkWidget *ScriptView; /**< a GtkTextView containing a scheme script */
