@@ -42,7 +42,10 @@ newclefify (DenemoObject * thechord, gint dclef)
 /**
  *  This function goes through a chord and checks to see how its notes are
  * laid out, and if it will have to push the display of any notes to the
- * "wrong" side of the staff as a result 
+ * "wrong" side of the stem as a result
+ * the result is to set the is_reversealigned field of the chord
+ * and the reversealign field of the note* structures that make up the chord.
+ * thechord must have type CHORD on entry. 
  */
 void
 findreversealigns (DenemoObject * thechord)
@@ -66,7 +69,8 @@ findreversealigns (DenemoObject * thechord)
 	     previous = current, prevnote = curnote, current = current->next)
 	  {
 	    curnote = (note *) current->data;
-	    if (prevnote->mid_c_offset == curnote->mid_c_offset - 1)
+	    if ((prevnote->mid_c_offset == curnote->mid_c_offset - 1) ||
+		(prevnote->mid_c_offset == curnote->mid_c_offset))
 	      {
 		((chord *) thechord->object)->is_reversealigned = TRUE;
 		curnote->reversealign = !prevnote->reversealign;
@@ -88,7 +92,8 @@ findreversealigns (DenemoObject * thechord)
 	     previous = current, prevnote = curnote, current = current->prev)
 	  {
 	    curnote = (note *) current->data;
-	    if (prevnote->mid_c_offset == curnote->mid_c_offset + 1)
+	    if ((prevnote->mid_c_offset == curnote->mid_c_offset + 1)||
+		(prevnote->mid_c_offset == curnote->mid_c_offset))
 	      {
 		curnote->reversealign = !prevnote->reversealign;
 		((chord *) thechord->object)->is_reversealigned = TRUE;
@@ -455,23 +460,39 @@ changenumdots (DenemoObject * thechord, gint number)
   set_basic_numticks (thechord);
 }
 
+static void 
+freenote(note *thenote) {
+#define FREE(field) if(thenote->field) g_string_free(thenote->field, TRUE);
+  FREE(display);
+  FREE(prefix);
+  FREE(postfix);
+#undef FREE
+  g_free(thenote);
+}
+
+
 /**
  * Free the current chord
  */
 void
 freechord (DenemoObject * thechord)
 {
-  g_list_foreach (((chord *) thechord->object)->notes, freeit, NULL);
+  g_list_foreach (((chord *) thechord->object)->notes, freenote, NULL);
   g_list_free (((chord *) thechord->object)->notes);
   g_list_free(((chord *) thechord->object)->dynamics);
   g_list_free(((chord *) thechord->object)->ornamentlist);
   if(((chord *) thechord->object)->lyric)
-    g_string_free(((chord *) thechord->object)->lyric, FALSE);
+    g_string_free(((chord *) thechord->object)->lyric, FALSE);//FIXME memory leak???? 
   /* tone_node does not belong to the chord but belongs instead to the pitch recognition system */
   if(((chord *) thechord->object)->is_figure && ((chord *) thechord->object)->figure)
-    g_string_free(((chord *) thechord->object)->figure, FALSE);
-  
-  g_free (thechord);
+    g_string_free(((chord *) thechord->object)->figure, FALSE);//FIXME memory leak???? 
+#define FREE(field) if(((chord *) thechord->object)->field)\
+         g_string_free(((chord *) thechord->object)->field, TRUE);
+  FREE(display);
+  FREE(prefix);
+  FREE(postfix);
+#undef FREE
+  g_free (thechord);//FIXME free thechord->object
 }
 
 /**
@@ -484,6 +505,7 @@ clone_chord (DenemoObject * thechord)
   DenemoObject *ret = (DenemoObject *) g_malloc0 (sizeof (DenemoObject));
   GList *curtone;
   note *newnote;
+  chord *curchord = (chord *) thechord->object;
   chord *clonedchord = (chord *) g_malloc0 (sizeof (chord));
   /* I'd use a g_list_copy here, only that won't do the deep copy of
    * the list data that I'd want it to */
@@ -496,7 +518,7 @@ clone_chord (DenemoObject * thechord)
    * A gpointer doesn't know the type it is so have to explictly 
    * copy the object */
 
-  memcpy ((chord *) clonedchord, (chord *) thechord->object, sizeof (chord));
+  memcpy ((chord *) clonedchord, curchord, sizeof (chord));
 
 #if 0
   //this would have to clone the list to work, else both notes share the same list and deleting one should
@@ -514,14 +536,28 @@ clone_chord (DenemoObject * thechord)
   clonedchord->is_figure = FALSE;
   clonedchord->lyric = NULL;
 
-
+#define CLONE(field) \
+      if(curchord->field && curchord->field->len)\
+        clonedchord->field = g_string_new(curchord->field->str);
+      CLONE(prefix);
+      CLONE(postfix);
+      CLONE(display);
+#undef CLONE
 
   clonedchord->notes = NULL;
   for (curtone = ((chord *) thechord->object)->notes;
        curtone; curtone = curtone->next)
     {
       newnote = (note *) g_malloc0 (sizeof (note));
-      memcpy (newnote, (note *) curtone->data, sizeof (note));
+      note *curnote = (note *) curtone->data;
+      memcpy (newnote, curnote, sizeof (note));
+#define CLONE(field) \
+      if(curnote->field && curnote->field->len)\
+        newnote->field = g_string_new(curnote->field->str);
+      CLONE(prefix);
+      CLONE(postfix);
+      CLONE(display);
+#undef CLONE
       clonedchord->notes = g_list_append (clonedchord->notes, newnote);
     }
   ret->object = (chord *) clonedchord;
