@@ -25,7 +25,7 @@ void modify_note (chord *thechord,  gint mid_c_offset, gint enshift, gint dclef)
 note* addtone (DenemoObject * mudelaobj, gint mid_c_offset, gint enshift,
 	      gint dclef);
 
-void removetone (DenemoObject * mudelaobj, gint mid_c_offset, gint dclef);
+gboolean removetone (DenemoObject * mudelaobj, gint mid_c_offset, gint dclef);
 
 void shiftpitch (DenemoObject * mudelaobj, gint mid_c_offset,
 		 gint is_sharpening);
