@@ -107,7 +107,9 @@ beamandstemdirhelper (DenemoScore * si)
 
 
 /**
- * Set si->current* variables
+ * Set si->current* variables from currentmeasurenum
+ * the current object is set to the first in the measure
+ * the selection is updated
  *
  */
 void
@@ -287,12 +289,20 @@ adjuststaffheight (DenemoScore * si, gint amount)
  *
  */
 void
-measureleft (DenemoGUI * gui)
+measureleft (DenemoScriptParam *param)
 {
+  DenemoGUI *gui = Denemo.gui;
+  DenemoScore *si = gui->si;
+  DenemoScriptParam dummy;
+  if(param==NULL)
+    param = &dummy;
+  param->status = FALSE;
+
   if (!gui->si->cursor_x && gui->si->currentmeasure->prev)
     {
       gui->si->currentmeasurenum--;
       isoffleftside (gui);
+      param->status = TRUE;
     }
   setcurrents (gui->si);
 }
@@ -302,13 +312,21 @@ measureleft (DenemoGUI * gui)
  *
  */
 void
-measureright (DenemoGUI * gui)
+measureright (DenemoScriptParam *param)
 {
+  DenemoGUI *gui = Denemo.gui;
+  DenemoScore *si = gui->si;
+  DenemoScriptParam dummy;
+  if(param==NULL)
+    param = &dummy;
+  param->status = FALSE;
+
   if (gui->si->currentmeasure->next)
     {
       gui->si->currentmeasurenum++;
       isoffrightside (gui);
       setcurrents (gui->si);
+      param->status = TRUE;
     }
 }
 
@@ -360,7 +378,6 @@ swapstaffs (GtkAction *action, gpointer param)
       temp = gui->si->currentstaff->data;
       if(temp->context==DENEMO_NONE ||
 	confirm("A context is set on this staff", "You will need to alter the staff->properties->context of this and the previous staff; Proceed?")) {
-	  
 	  gui->si->currentstaff->data = gui->si->currentstaff->prev->data;
 	  gui->si->currentstaff->prev->data = temp;
 	  gui->si->currentstaffnum--;
@@ -369,6 +386,7 @@ swapstaffs (GtkAction *action, gpointer param)
 	  setcurrents (gui->si);
 	  move_viewport_up (gui);
 	  score_status(gui, TRUE);
+	  displayhelper(gui);
 	  return TRUE;
 	}
     }
@@ -399,6 +417,7 @@ splitstaffs (GtkAction *action, gpointer param)
       setcurrents (gui->si);
       move_viewport_up (gui);
       score_status(gui, TRUE);
+      displayhelper(gui);
       return TRUE;
     }
   else
@@ -426,6 +445,7 @@ joinstaffs (GtkAction *action, gpointer param)
       setcurrents (gui->si);
       move_viewport_up (gui);
       score_status(gui, TRUE);
+      displayhelper(gui);
       return TRUE;
     }
   else
@@ -436,23 +456,30 @@ joinstaffs (GtkAction *action, gpointer param)
 
 /**
  * Move si->currentstaff up an one voice, return TRUE if successful
- *
+ * param is NULL for interactive calls, otherwise status is returned in param->status;
  */
 gboolean
-voiceup (DenemoGUI * gui)
+voiceup (DenemoScriptParam *param)
 {
+  DenemoGUI *gui = Denemo.gui;
+  DenemoScore *si = gui->si;
+  DenemoScriptParam dummy;
+  if(param==NULL)
+    param = &dummy;
+  param->status = FALSE;
   if(!gui->si->currentstaff)
-    return FALSE;
+    return param->status = FALSE;
   if (gui->si->currentstaff && (((DenemoStaff *)(gui->si->currentstaff->data))->voicenumber==2))  {
       gui->si->currentstaffnum--;
       gui->si->currentstaff = gui->si->currentstaff->prev;
       setcurrentprimarystaff (gui->si);
       setcurrents (gui->si);
       move_viewport_down (gui);
-      return TRUE;
+      return param->status = TRUE;
     } else
-      warningdialog("This is the first voice");
-  return FALSE;
+      if(param==&dummy)//is interactive
+	warningdialog("This is the first voice");
+  return param->status = FALSE;
 }
 
 /**
@@ -460,23 +487,31 @@ voiceup (DenemoGUI * gui)
  *
  */
 gboolean
-staffup (DenemoGUI * gui)
+staffup (DenemoScriptParam *param)
 {
+  DenemoGUI *gui = Denemo.gui;
+  DenemoScore *si = gui->si;
+  DenemoScriptParam dummy;
+  if(param==NULL)
+    param = &dummy;
+  param->status = FALSE;
   if(!gui->si->currentstaff)
-    return FALSE;
+    return param->status = FALSE;
   while (((DenemoStaff *)(gui->si->currentstaff->data))->voicenumber!=1)
-    voiceup(gui);
+    voiceup(param);//FIXME check param->status
   if (gui->si->currentstaff->prev)
     {
       gui->si->currentstaffnum--;
       gui->si->currentstaff = gui->si->currentstaff->prev;
       setcurrentprimarystaff (gui->si);
       setcurrents (gui->si);
+      gtk_widget_draw (gui->scorearea, NULL);//KLUDGE FIXME gets cursorclef set
       move_viewport_up (gui);
-      return TRUE;
-    }else
-      warningdialog("This is the first staff");
-  return FALSE;
+      return param->status = TRUE;
+    } else
+      if(param==&dummy)//is interactive
+	warningdialog("This is the first staff");
+  return param->status = FALSE;
 }
 
 
@@ -485,41 +520,59 @@ staffup (DenemoGUI * gui)
  *
  */
 gboolean
-voicedown (DenemoGUI * gui)
+voicedown (DenemoScriptParam *param)
 {
+  DenemoGUI *gui = Denemo.gui;
+  DenemoScore *si = gui->si;
+  DenemoScriptParam dummy;
+  if(param==NULL)
+    param = &dummy;
+  param->status = FALSE;
   if(!gui->si->currentstaff)
-    return FALSE;
+    return param->status = FALSE;
   if (gui->si->currentstaff->next && ((DenemoStaff *)(gui->si->currentstaff->next->data))->voicenumber==2) {
       gui->si->currentstaffnum++;
       gui->si->currentstaff = gui->si->currentstaff->next;
       setcurrentprimarystaff (gui->si);
       setcurrents (gui->si);
       move_viewport_down (gui);
-      return TRUE;
+      return param->status = TRUE;
     } else
+      if(param==&dummy)//is interactive
       warningdialog("This is the last voice");
-  return FALSE;
+  return param->status = FALSE;
 }
 /**
  * Move si->currentstaff down an one staff/voice, return TRUE if successful
  *
  */
 gboolean
-staffdown (DenemoGUI * gui)
+staffdown (DenemoScriptParam *param)
 {
+  DenemoGUI *gui = Denemo.gui;
+  DenemoScore *si = gui->si;
+  DenemoScriptParam dummy;
+  if(param==NULL)
+    param = &dummy;
+  param->status = FALSE;
+
   if(!gui->si->currentstaff)
-    return FALSE;
+    return param->status = FALSE;
   while (gui->si->currentstaff->next && ((DenemoStaff *)(gui->si->currentstaff->next->data))->voicenumber==2)
-    voicedown(gui);
+    voicedown(param);//FIXME
   if (gui->si->currentstaff->next)
     {
       gui->si->currentstaffnum++;
       gui->si->currentstaff = gui->si->currentstaff->next;
       setcurrentprimarystaff (gui->si);
       setcurrents (gui->si);
+      gtk_widget_draw (gui->scorearea, NULL);//KLUDGE FIXME gets cursorclef set
       move_viewport_down (gui);
-    }else
+      return param->status = TRUE;
+    } else
+      if(param==&dummy)//is interactive
       warningdialog("This is the last staff");
+  return param->status = FALSE;
 }
 
 
@@ -529,9 +582,14 @@ staffdown (DenemoGUI * gui)
  *
  */
 void
-cursorleft (DenemoGUI * gui)
+cursorleft (DenemoScriptParam *param)
 {
+  DenemoGUI *gui = Denemo.gui;
   DenemoScore *si = gui->si;
+  DenemoScriptParam dummy;
+  if(param==NULL)
+    param = &dummy;
+  param->status = FALSE;
   g_debug("cursorleft: cursorpos %d\n", si->cursor_x);
   if (!si->cursor_x)
     {
@@ -551,6 +609,7 @@ cursorleft (DenemoGUI * gui)
 	  si->cursor_x = g_list_length ((objnode *) si->currentmeasure->data);
 	  /* Despite appearances, there is not an off-by-one error in the
 	   * preceding command */
+	  param->status = TRUE;
 	}
     }
   else if (si->cursor_appending)
@@ -558,6 +617,7 @@ cursorleft (DenemoGUI * gui)
       /* Can back off from appending */
       si->cursor_appending = FALSE;
       si->cursor_x--;
+      param->status = TRUE;
     }
   else
     {
@@ -566,6 +626,7 @@ cursorleft (DenemoGUI * gui)
 	{
 	  si->currentobject = si->currentobject->prev;
 	  si->cursor_x--;
+	  param->status = TRUE;
 	}
     }
   calcmarkboundaries (si);
@@ -574,12 +635,17 @@ cursorleft (DenemoGUI * gui)
 
 /**
  * move the cursor one position to the right
- *
+ * sets param->status TRUE if cursor could still move right.
  */
-void
-cursorright (DenemoGUI * gui)
+gboolean
+cursorright (DenemoScriptParam *param)
 {
+  DenemoGUI *gui = Denemo.gui;
   DenemoScore *si = gui->si;
+  DenemoScriptParam dummy;
+  if(param==NULL)
+    param = &dummy;
+  param->status = FALSE;
   if (si->cursor_appending && si->currentmeasure->next)
     {
       /* Go to the next measure */
@@ -606,19 +672,28 @@ cursorright (DenemoGUI * gui)
 	  si->currentobject = si->currentobject->next;
 	  si->cursor_x++;
 	}
+      else return param->status = FALSE;
     }
   calcmarkboundaries (si);
   write_status(gui);
+  return (param->status = (si->currentobject || (!si->cursor_appending) || si->currentmeasure->next));
 }
 
 /**
  * Move the cursor up one position 
  */
 void
-cursorup (DenemoGUI * gui)
+cursorup (DenemoScriptParam *param)
 {
+  DenemoGUI *gui = Denemo.gui;
+  DenemoScore *si = gui->si;
+  DenemoScriptParam dummy;
+  if(param==NULL)
+    param = &dummy;
+  param->status = FALSE;
   gui->si->cursor_y++;
   gui->si->staffletter_y = (gui->si->staffletter_y + 1) % 7;
+  param->status = TRUE;//FIXME introduce some range boundaries, settable by user for instrument ranges.
   //g_print ("Cursor Y Position %d\n", gui->si->cursor_y);
 }
 
@@ -626,10 +701,18 @@ cursorup (DenemoGUI * gui)
  * Move the cursor down one position 
  */
 void
-cursordown (DenemoGUI * gui)
+cursordown (DenemoScriptParam *param)
 {
+  DenemoGUI *gui = Denemo.gui;
+  DenemoScore *si = gui->si;
+  DenemoScriptParam dummy;
+  if(param==NULL)
+    param = &dummy;
+  param->status = FALSE;
+
   gui->si->cursor_y--;
   gui->si->staffletter_y = (gui->si->staffletter_y + 6) % 7;
+  param->status = TRUE;//FIXME introduce some range boundaries, settable by user for instrument ranges.
   //g_print ("Cursor Y Position %d\n", gui->si->cursor_y);
 }
 
@@ -649,10 +732,10 @@ shiftcursor (DenemoGUI  *gui, gint note_value)
   gui->si->cursor_y = jumpcursor (gui->si->cursor_y, oldstaffletter_y,
 				  gui->si->staffletter_y);
   int mid_c_offset = gui->si->cursor_y;
-  if((gui->mode & INPUTRHYTHM)&&!(gui->mode&INPUTINSERT)){
-    warningdialog("Rhythm only mode - enter durations, not notes!");
-    return;
-  }
+//  if((gui->mode & INPUTRHYTHM)&&!(gui->mode&INPUTINSERT)){
+//    warningdialog("Rhythm only mode - enter durations, not notes!");
+//    return;
+//  }
      
   /* in edit mode edit the current note name */
   if((gui->mode & INPUTEDIT) && gui->si->currentobject) {
@@ -714,10 +797,10 @@ insert_rhythm_pattern(DenemoGUI  *gui) {
 	  insertion_point (gui->si);
 	  gui->si->cursoroffend = FALSE;
 	  if(!code_is_a_duration(modifier_code(h->data)))
-	    cursorleft(gui);
+	    cursorleft(NULL);
 	  ((GtkFunction)h->data)(gui);
 	  if(!code_is_a_duration(modifier_code(h->data)))
-	    cursorright(gui);
+	    cursorright(NULL);
 	  displayhelper(gui);
 	}
 	h = ((RhythmElement*)g->data)->functions;
@@ -740,7 +823,7 @@ insert_rhythm_pattern(DenemoGUI  *gui) {
 void
 insertion_point (DenemoScore * si)
 {
-
+  gtk_widget_draw(Denemo.gui->scorearea, NULL);//FIXME efficiency????
   gboolean next_measure;
 
   /* First, check to see if the insertion'll cause the cursor to
@@ -766,6 +849,8 @@ insertion_point (DenemoScore * si)
 	}
       else
 	si->currentmeasure = si->currentmeasure->next;
+      if(Denemo.gui->mode&(INPUTRHYTHM))
+	gdk_beep();
       /* Now the stuff that needs to be done for each case */
       si->currentmeasurenum++;
       si->currentobject = (objnode *) si->currentmeasure->data;
@@ -790,9 +875,9 @@ dnm_insertchord (DenemoGUI * gui, gint duration, input_mode mode,
   DenemoScore *si = gui->si;
   DenemoObject *mudela_obj_new;
   int prognum;
-  DenemoStaff *curstaffstruct;
+  
 
-  if((mode & INPUTEDIT) && !si->cursor_appending) {
+  if((mode & INPUTEDIT) && !si->cursor_appending && !(mode & INPUTRHYTHM)) {
     changeduration(si, duration);
     return;
   }
@@ -806,8 +891,8 @@ dnm_insertchord (DenemoGUI * gui, gint duration, input_mode mode,
   if ((mode & INPUTBLANK) || (gui->mode & INPUTBLANK))
     mudela_obj_new->isinvisible = TRUE;
 
-  if (si->is_grace_mode)
-    ((chord *) mudela_obj_new->object)->is_grace = TRUE;
+  //  if (si->is_grace_mode)
+  //   ((chord *) mudela_obj_new->object)->is_grace = TRUE;
 
 
   /* Insert the new note into the score.  Note that while we may have
@@ -816,10 +901,20 @@ dnm_insertchord (DenemoGUI * gui, gint duration, input_mode mode,
      don't need to invoke that here.  */
 
   object_insert (gui, mudela_obj_new);
-  curstaffstruct = (DenemoStaff *) si->currentstaff->data;
-  prognum = select_program (curstaffstruct->midi_instrument->str);
-  /*playnotes (si->prefs->immediateplayback, *(chord *) mudela_obj_new->object,
-     prognum); */
+ 
+  if (gui->mode&(INPUTRHYTHM)) {
+    if(rest)
+      play_pitch(64.0*(1+duration), 60.0/(4.0*gui->si->tempo*(1<<duration)), 0.1, 2);//FIXME make a distinct noise
+      else
+	play_pitch(64.0*(1+duration), 60.0/(gui->si->tempo*(1<<duration)), 0.2, 1);
+  } else {
+    if(Denemo.gui->input_source==INPUTKEYBOARD) {
+      DenemoStaff *curstaffstruct = (DenemoStaff *) si->currentstaff->data;
+      prognum = select_program (curstaffstruct->midi_instrument->str);
+      playnotes (Denemo.prefs.immediateplayback, *(chord *) mudela_obj_new->object,
+		 prognum);
+    }
+  }
 }
 
 /**
@@ -892,7 +987,7 @@ insertgrace (DenemoGUI * gui)
   si->currentobject =
     g_list_nth ((objnode *) si->currentmeasure->data, si->cursor_x);
   si->cursor_appending = FALSE;
-  si->is_grace_mode = TRUE;
+  // useless, never set false again ... si->is_grace_mode = TRUE;
 }
 
 /**
@@ -926,7 +1021,7 @@ tonechange (DenemoScore * si, gboolean remove)
   declarecurmudelaobj;
   int prognum;
   gboolean ret = FALSE;
-  DenemoStaff *curstaffstruct;
+  
   if (curmudelaobj && curmudelaobj->type == CHORD)
     {
       if (remove == TRUE)
@@ -940,10 +1035,12 @@ tonechange (DenemoScore * si, gboolean remove)
 	  g_free (curmudelaobj->user_string);
 	  curmudelaobj->user_string = NULL;
 	}
-      curstaffstruct = (DenemoStaff *) si->currentstaff->data;
-      prognum = select_program (curstaffstruct->midi_instrument->str);
-      /*playnotes (si->prefs->immediateplayback,
-       *(chord *) curmudelaobj->object, prognum);*/
+      if(Denemo.gui->input_source==INPUTKEYBOARD) {
+	DenemoStaff *curstaffstruct = (DenemoStaff *) si->currentstaff->data;
+	prognum = select_program (curstaffstruct->midi_instrument->str);
+	playnotes (Denemo.prefs.immediateplayback,
+	 *(chord *) curmudelaobj->object, prognum);
+      }
     }
   return ret;
 }
@@ -966,9 +1063,11 @@ displayhelper (DenemoGUI * gui)
   nudgerightward (gui);
   set_bottom_staff(gui);
   write_status(gui);
+
+#if 0
   if ((gui->mode&(INPUTRHYTHM)) && si->currentobject && (((DenemoObject*)(si->currentobject->data))->type==CHORD)&& ((DenemoObject*)(si->currentobject->data))->starttickofnextnote>= WHOLE_NUMTICKS * si->cursortime1/si->cursortime2)
     gdk_beep(); //Signal new measures in Edit mode to catch out of step entry
-
+#endif
   /*gtk_widget_draw (gui->scorearea, NULL);*/
   gtk_widget_queue_draw (gui->scorearea);
 }
@@ -985,7 +1084,7 @@ incrementenshift (DenemoGUI * gui, gint direction)
   DenemoScore *si = gui->si;
   declarecurmudelaobj;
   int prognum;
-  DenemoStaff *curstaffstruct;
+  
   if (curmudelaobj && curmudelaobj->type == CHORD)
     {
       shiftpitch (curmudelaobj, si->cursor_y, direction>0);
@@ -998,11 +1097,13 @@ incrementenshift (DenemoGUI * gui, gint direction)
 	  g_free (curmudelaobj->user_string);
 	  curmudelaobj->user_string = NULL;
 	}
-      curstaffstruct = (DenemoStaff *) si->currentstaff->data;
-      prognum = select_program (curstaffstruct->midi_instrument->str);
-      /* playnotes (si->prefs->immediateplayback,
-       *(chord *) curmudelaobj->object, prognum);*/
+      if(Denemo.gui->input_source==INPUTKEYBOARD) {
+	DenemoStaff *curstaffstruct = (DenemoStaff *) si->currentstaff->data;
+	prognum = select_program (curstaffstruct->midi_instrument->str);
 
+	playnotes (Denemo.prefs.immediateplayback,
+		   *(chord *) curmudelaobj->object, prognum);
+      }
       unre_data *data = (unre_data *) g_malloc (sizeof (unre_data));
       data->object = curmudelaobj;
       data->position = si->cursor_x;
@@ -1024,7 +1125,6 @@ setenshift (DenemoScore * si, gint enshift)
 {
   declarecurmudelaobj;
   int prognum;
-  DenemoStaff *curstaffstruct;
   if (curmudelaobj && curmudelaobj->type == CHORD)
     {
       changeenshift (curmudelaobj, si->cursor_y, enshift);
@@ -1037,11 +1137,13 @@ setenshift (DenemoScore * si, gint enshift)
 	  g_free (curmudelaobj->user_string);
 	  curmudelaobj->user_string = NULL;
 	}
-      curstaffstruct = (DenemoStaff *) si->currentstaff->data;
-      prognum = select_program (curstaffstruct->midi_instrument->str);
-      /* playnotes (si->prefs->immediateplayback,
-       *(chord *) curmudelaobj->object, prognum);*/
 
+      if(Denemo.gui->input_source==INPUTKEYBOARD) {
+	DenemoStaff *curstaffstruct = (DenemoStaff *) si->currentstaff->data;
+	prognum = select_program (curstaffstruct->midi_instrument->str);
+	playnotes (Denemo.prefs.immediateplayback,
+		   *(chord *) curmudelaobj->object, prognum);
+      }
       unre_data *data = (unre_data *) g_malloc (sizeof (unre_data));
       data->object = curmudelaobj;
       data->position = si->cursor_x;
@@ -1100,6 +1202,8 @@ changedots (DenemoScore * si, gint amount)
 
   if (curmudelaobj && curmudelaobj->type == CHORD)
     {
+      if (Denemo.gui->mode&(INPUTRHYTHM))
+	  play_pitch(440.0, 0.2, 0.2, 1);
       changenumdots (curmudelaobj, amount);
 
       if (curmudelaobj->user_string)
@@ -1185,7 +1289,7 @@ delete_staff_before (GtkAction *action, gpointer param)
 {
   DenemoGUI *gui = Denemo.gui;
   DenemoScore *si = gui->si;
-  if (staffup(gui)) {
+  if (staffup(param)) {
     deletestaff (gui, TRUE);
   }
 }
@@ -1201,7 +1305,7 @@ delete_staff_after (GtkAction *action, gpointer param)
 {
   DenemoGUI *gui = Denemo.gui;
   DenemoScore *si = gui->si;
-  if (staffdown(gui)) {
+  if (staffdown(param)) {
     deletestaff (gui, TRUE);
   }
 }
@@ -1500,7 +1604,7 @@ toend (GtkAction *action, gpointer param)
     g_list_length (((DenemoStaff *) gui->si->currentstaff->data)->measures);
   setcurrents (gui->si);
   tolastobject(gui);
-  cursorright(gui);
+  cursorright(param);
   find_leftmost_allcontexts (gui->si);
   update_hscrollbar (gui);
   displayhelper (gui);
