@@ -7,9 +7,14 @@
  * (c) 1999-2005 Matthew Hiller
  */
 
+#ifndef __CONTEXTS_H__
+#define __CONTEXTS_H__
 #include <denemo/denemo.h>
 
 void find_leftmost_staffcontext (DenemoStaff * curstaffstruct,
 				 DenemoScore *si);
 
 void find_leftmost_allcontexts (DenemoScore *si);
+gint 
+find_prevailing_clef(DenemoScore *si);
+#endif /* __CONTEXTS_H__ */
