@@ -113,7 +113,7 @@ draw_object (objnode * curobj, gint x, gint y,
   static GdkGC *redgc = NULL;
   static GdkGC *greengc = NULL;
   static gboolean init=FALSE;
-  static GdkColor white, black, blue, green;
+  static GdkColor white, black, blue, green, yellow;
   if(!init) {
     gdk_color_parse ("white", &white);
     gdk_colormap_alloc_color (gdk_colormap_get_system (), &white, TRUE, TRUE);
@@ -123,6 +123,8 @@ draw_object (objnode * curobj, gint x, gint y,
     gdk_colormap_alloc_color (gdk_colormap_get_system (), &blue, TRUE, TRUE);
     gdk_color_parse ("green", &green);
     gdk_colormap_alloc_color (gdk_colormap_get_system (), &green, TRUE, TRUE);
+    gdk_color_parse ("yellow", &yellow);
+    gdk_colormap_alloc_color (gdk_colormap_get_system (), &yellow, TRUE, TRUE);
     init = TRUE;
   }
   itp->highy = itp->lowy = 0;
@@ -140,6 +142,7 @@ draw_object (objnode * curobj, gint x, gint y,
     greengc = gcs_greengc ();
   /* Should we set cursor-context info before drawing? */
 
+  /************ FIXME the drawing is side-effecting the DenemoScore si here *******************/
   if (si->currentobject == curobj)
     {
       si->cursorclef = itp->clef;
@@ -151,7 +154,7 @@ draw_object (objnode * curobj, gint x, gint y,
     
     GdkColor *thecolor;
     if(mudelaitem->type==CHORD && ((chord *) mudelaitem->object)->tone_node)
-      thecolor = &green;
+      thecolor = &yellow;
     else
       thecolor =/* (mudelaitem->isinvisible) ? &white :*/ itp->mark?&blue:&black;
     gdk_gc_set_foreground (blackgc, thecolor);
@@ -649,6 +652,7 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
   struct infotopass itp;
   static GdkGC *blackgc = NULL;
   static GdkGC *bluegc;
+  static GdkGC *graygc;
   GList *mwidthiterator;
   gboolean repeat = FALSE;
   DenemoScore *si = gui->si;
@@ -658,11 +662,12 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
     {
       blackgc = gcs_blackgc ();
       bluegc = gcs_bluegc ();
+      graygc = gcs_graygc ();
     }
 
   /* Initialize some fields in itp */
 
-  itp.widget = widget;
+  itp.widget = widget;// this is used for widget->style, it is spuriously passed to the macro bitmaphelper
   itp.slur_stack = NULL;
   itp.hairpin_stack = NULL;
   itp.haslyrics = FALSE;
@@ -707,7 +712,7 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
 	   
 	   if (!mwidthiterator->next) /* Last measure - draw double-barline */
 	     x -= 3;
-	   gdk_draw_line (gui->pixmap, blackgc, x, top, x, y);
+	   gdk_draw_line (gui->pixmap, graygc, x, top, x, y);
 	   
 	   if (!mwidthiterator->next)
 	     {
@@ -759,9 +764,11 @@ scorearea_expose_event (GtkWidget * widget, GdkEventExpose * event)
 DenemoGUI *gui = Denemo.gui;
   do{
   /* Clear the backing pixmap */
-  if(pitch_entry_active(gui)) {
+  if(Denemo.gui->input_source!=INPUTKEYBOARD &&
+     (Denemo.prefs.overlays || (Denemo.gui->input_source==INPUTAUDIO))
+     && pitch_entry_active(gui)) {
     gdk_draw_rectangle (gui->pixmap,
-			gcs_graygc(),
+			gcs_lightbluegc(),
 			TRUE,
 			0, 0,
 			widget->allocation.width, widget->allocation.height);
