@@ -18,18 +18,39 @@ void
 draw_lily_dir (GdkPixmap * pixmap, GdkGC * gc, GdkFont * font,
 	       gint xx, gint y, gint highy, gint lowy, DenemoObject * theobj, gboolean selected)
 {
+  lilydirective *lily = ((lilydirective *) theobj->object);
   PangoContext *context =
     gdk_pango_context_get_for_screen (gdk_drawable_get_screen (pixmap));
   PangoLayout *layout = pango_layout_new (context);
   PangoFontDescription *desc = pango_font_description_from_string (FONT);
-  gchar first = *(((lilydirective *) theobj->object)->directive->str);
-  if( first == '%' || first == '^' || first == '_' ) { //display comments, and markup above and below
+  gchar *first = (lily->directive->str);
+
+  if(lily->display) {  //store display position x,y as well
     pango_layout_set_text (layout,
-			   ((lilydirective *) theobj->object)->directive->str+1,
+			   lily->display->str,
 			   -1);
     pango_layout_set_font_description (layout, desc);
-    pango_font_description_free (desc); 
-    gdk_draw_layout (pixmap, selected?gcs_bluegc():gc, xx, first=='_'?y+lowy+20:y-highy-20 /*y+(first=='_'?STAFF_HEIGHT+20:-20)*/, layout);
+    gdk_draw_layout (pixmap, selected?gcs_bluegc():gc, xx/*+display x */, y+lowy+20/*+display y */, layout);
   }
-  gdk_draw_rectangle (pixmap, selected?gcs_bluegc():gcs_greengc(), TRUE, xx, y, 2, STAFF_HEIGHT/2);
+#if 1
+  else
+  //do this by creating a display field
+
+  if( *first == '%' || *first == '^' || *first == '_' ) { //display comments, and markup above and below
+    pango_layout_set_text (layout,
+			   first+1,
+			   -1);
+    pango_layout_set_font_description (layout, desc);
+     
+    gdk_draw_layout (pixmap, selected?gcs_bluegc():gc, xx, *first=='_'?y+lowy+20:y-highy-20 /*y+(*first=='_'?STAFF_HEIGHT+20:-20)*/, layout);
+  }
+#endif
+
+
+
+   pango_font_description_free (desc); 
+
+
+
+  gdk_draw_rectangle (pixmap, selected?gcs_bluegc():gcs_greengc(), TRUE, xx, y+3*STAFF_HEIGHT/2, 2, STAFF_HEIGHT/2);
 }
