@@ -361,7 +361,7 @@ static void output_lyric(GString *lyrics, chord *pchord, gboolean *pis_syllable,
       else
 	lyrics = g_string_append (lyrics, "__ ");
     } else
-      lyrics = g_string_append (lyrics, "--");//FIXME is this right?
+      lyrics = g_string_append (lyrics, " - ");//FIXME is this right?
 }
 
 
@@ -372,13 +372,17 @@ static void output_lyric(GString *lyrics, chord *pchord, gboolean *pis_syllable,
 static void
 output_figured_bass (DenemoScore * si, GString *figures, chord * pchord)
 {
+  static gboolean continuation = FALSE;
+  gboolean continuation_finishing = FALSE;
+  static GString *last_figure;// for continuations
   gint duration = internaltomuduration (pchord->baseduration);
   gint numdots = pchord->numdots;
   GString *fig_str;		/*working copy of figures string stored in pchord */
   char *str;			/* pointer into the figure string fig_str */
   gint num_groups = 1;		/* number of groups of figures */
+  if(!last_figure)
+    last_figure = g_string_new("");
 
-  figures = g_string_append (figures, "<");
   if (pchord->figure == NULL
       ||
       (((GString
@@ -388,8 +392,18 @@ output_figured_bass (DenemoScore * si, GString *figures, chord * pchord)
     fig_str =
       g_string_new (((GString
 		      *) ((chord *) pchord->figure))->str);
-		      
-		    
+
+  if(*fig_str->str == '~') {
+    if(!continuation) {
+      figures = g_string_append (figures, " \\set useBassFigureExtenders = ##t ");
+      continuation = TRUE;
+    }
+    if(last_figure->len) {
+      figures = g_string_append (figures, "<");
+      figures = g_string_append (figures, last_figure->str);
+    }
+  } else
+    figures = g_string_append (figures, "<");	    
 
   /* multiple figures are separated by a FIGURES_SEP char,
      output these at subdivisions of the duration */
@@ -408,9 +422,18 @@ output_figured_bass (DenemoScore * si, GString *figures, chord * pchord)
     {
     default:
     case 1:
-      figures = g_string_append (figures, fig_str->str);
+        if(*fig_str->str != '~') {
+	if(continuation)
+	  continuation_finishing = TRUE;
+	figures = g_string_append (figures, fig_str->str);
+	g_string_assign(last_figure, fig_str->str);
+      }
       figures = g_string_append (figures, ">");
       append_duration (figures, duration, numdots);
+      if(continuation_finishing) {
+	figures = g_string_append (figures, "\\set useBassFigureExtenders = ##f ");
+	continuation = FALSE;
+      }
       break;
       /* Each group of figures is assigned a duration to
          achieve a normal looking output */
@@ -535,8 +558,9 @@ output_fakechord (DenemoScore * si, GString *fakechord, chord * pchord)
     fig_str = g_string_new (" r");	/* the no-fakechord figure */
   else {
     fig_str =
-      g_string_new (((GString
-		      *) ((chord *) pchord->fakechord))->str);
+       g_string_ascii_down (
+          g_string_new (((GString
+ 		         *) ((chord *) pchord->fakechord))->str));
   }
   if (pchord->fakechord_extension == NULL)
     extension = NULL;
@@ -668,11 +692,23 @@ insert_editable (GString **pdirective, gchar *original, GtkTextIter *iter, gchar
   gtk_text_buffer_insert_with_tags_by_name (gui->textbuffer, iter, " ", -1, HIGHLIGHT, invisibility, NULL);
 }
 
+static gint
+brace_count(gchar *str) {
+  gint ret;
+  for(ret=0;*str;str++) {
+    if(*str=='{') ret++;
+    if(*str=='}') ret--;
+  }
+  return ret;
+}
+
+
 /**
  * generate the lilypond for the DenemoObject curobj
  * the state of the prevailing duration, clef keysignature are updated and returned.
+ * returns the excess of open braces "{" created by this object.
  */
-static void
+static gint
 generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, DenemoObject * curobj, 
 		       GtkTextChildAnchor *objanc,		       
 		       gint * pprevduration, gint * pprevnumdots,
@@ -699,11 +735,12 @@ generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, D
   gint octave, enshift;
   gint noteheadtype;
   gint mid_c_offset;
-
+  gint open_braces=0;/* keep track of excess open braces "{" so as to ensure they are closed */
 
   GString *dynamic_string = NULL;
 
-  if(curobj->type==LILYDIRECTIVE){    
+  if(curobj->type==LILYDIRECTIVE){
+    open_braces += brace_count( ((lilydirective *) curobj->object)->directive->str);
     gtk_text_buffer_insert_with_tags_by_name (gui->textbuffer, iter,  ((lilydirective *) curobj->object)->directive->str, -1, "bold", invisibility, NULL);// Make it editable as well, the rest not...
     GtkTextChildAnchor *endanc  = gtk_text_buffer_create_child_anchor (gui->textbuffer, iter);
     GtkTextIter back;
@@ -728,6 +765,10 @@ generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, D
 	numdots = pchord->numdots;
 	is_chordmode = FALSE;
 
+	if(pchord->prefix && pchord->prefix->len) {
+	  open_braces += brace_count(pchord->prefix->str); 
+	  insert_editable(&pchord->prefix, pchord->prefix->str, iter, invisibility, gui);
+	}	    
 	if (!pchord->notes)
 	  {			/* A rest */
 	    if (!curobj->isinvisible)
@@ -747,11 +788,10 @@ generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, D
 	    else
 	      {
 		g_string_append_printf (ret, "\\skip ");
-		/* only in this case do we explicitly
-		   note the duration */
+	
 		g_string_append_printf (ret, "%d", duration);
-		prevduration = duration;
-		prevnumdots = numdots;
+		prevduration = -1;
+		prevnumdots = -1;
 		for (j = 0; j < numdots; j++)
 		  g_string_append_printf (ret, ".");
 	      }
@@ -762,7 +802,7 @@ generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, D
 	    GList *tmpornament;
 	    if (!curobj->isinvisible)
 	      {
-		if (pchord->notes->next  || ((note *) (pchord->notes)->data)->directive )//multinote chord, or treat as such
+		if (pchord->notes->next || pchord->chordize )//multinote chord
 		  {
 		    is_chordmode = TRUE;
 		    g_string_append_printf (ret, "<");
@@ -773,7 +813,9 @@ generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, D
 		  {
 		    note *curnote = (note *) notenode->data;
 		    noteheadtype = curnote->noteheadtype;
-
+		    if(curnote->prefix ) {
+		      insert_editable(&curnote->prefix, curnote->prefix->len?curnote->prefix->str:" ", iter, invisibility, gui);
+		    }
 		    switch (noteheadtype)
 		      {
 		      case DENEMO_NORMAL_NOTEHEAD:
@@ -828,20 +870,21 @@ generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, D
 
 
 		    outputret;
-		    if(curnote->directive ) {
-		      insert_editable(&curnote->directive, curnote->directive->len?curnote->directive->str:" ", iter, invisibility, gui);
+		    if(curnote->postfix ) {
+		      insert_editable(&curnote->postfix, curnote->postfix->len?curnote->postfix->str:" ", iter, invisibility, gui);
 		    }
 		    else
 		      if (notenode->next)
 			output(" ");
 		  }		/* End notes in chord loop */
 
-		if (pchord->notes->next ||  ((note *) (pchord->notes)->data)->directive) //multi-note chord
+		if (pchord->notes->next  || pchord->chordize) //multi-note chord
 		  g_string_append_printf (ret, ">");
 	      } //end of note(s) that is(are) not invisible
-	    else //invisible note
+	    else //invisible note - does this case exist??
 	      {
 		g_string_append_printf (ret, "s");
+		
 	      }
 	    if (duration != prevduration || numdots != prevnumdots)
 	      {
@@ -852,7 +895,6 @@ generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, D
 		for (j = 0; j < numdots; j++)
 		  g_string_append_printf (ret, ".");
 	      }
-
 	    if (pchord->dynamics && (pchord->notes->next==NULL))
 	      {
 		dynamic_string = (GString *) pchord->dynamics->data;
@@ -944,6 +986,13 @@ generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, D
 	      g_string_append_printf (ret, "(");
 	    if (pchord->is_tied)
 	      g_string_append_printf (ret, " ~");
+
+	    outputret;
+	    if(pchord->postfix && pchord->postfix->len) {
+	      open_braces += brace_count(pchord->postfix->str);
+	      insert_editable(&pchord->postfix, pchord->postfix->str, iter, invisibility, gui);
+	    }
+	      
 	    /* do this in caller                    g_string_append_printf (ret, " "); */
 	  } /* End of else chord with note(s) */
 	break;
@@ -1071,7 +1120,7 @@ generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, D
     *pkeyname = keyname;
     *pcur_stime1 = cur_stime1;
     *pcur_stime2 = cur_stime2;
-
+    return open_braces;
 }
 
 /* create and insertion point and button for the next piece of music */
@@ -1196,7 +1245,7 @@ outputStaff (DenemoGUI *gui, DenemoScore * si, DenemoStaff * curstaffstruct,
   gint curmeasurenum;// count of measures printed
   gint measurenum; //count of measures from start of staff starting at 1
   gint objnum;//count of objects in measure starting at 1
-
+  gint open_braces;//Keep track of the number of open brace "{" chars in the music, in case of imbalance.
   GString *str = g_string_new("");
   GString * lyrics = g_string_new("");
   GString * figures = g_string_new("");
@@ -1284,13 +1333,10 @@ outputStaff (DenemoGUI *gui, DenemoScore * si, DenemoStaff * curstaffstruct,
   curmeasure = curstaffstruct->measures;
   if(!end)
     end = g_list_length (curmeasure);
-
-
-
   /* Now each measure */
   if (start)
     curmeasure = g_list_nth (curmeasure, start - 1);
-
+  open_braces = 0;//keep track of excess open braces "{"
   for (measurenum = MAX (start, 1); curmeasure && measurenum <= end;
        curmeasure = curmeasure->next, measurenum++)
     {
@@ -1319,7 +1365,7 @@ outputStaff (DenemoGUI *gui, DenemoScore * si, DenemoStaff * curstaffstruct,
       for (objnum=1, curobjnode = (objnode *) curmeasure->data;/* curobjnode NULL checked at end */;
 	   curobjnode = curobjnode->next, objnum++)
 	{
-
+	  curobj=NULL;//avoid random values for debugabililty
 	  if(objnum>=firstobj && objnum<=lastobj) {
 
 	  if(curobjnode) {
@@ -1344,7 +1390,7 @@ outputStaff (DenemoGUI *gui, DenemoScore * si, DenemoStaff * curstaffstruct,
 	  gtk_text_buffer_apply_tag_by_name(gui->textbuffer, INEDITABLE, &back, &iter);
 	  gtk_text_buffer_apply_tag_by_name(gui->textbuffer, "system_invisible", &back, &iter);
 
-	  generate_lily_for_obj (gui, &iter, invisibility, curobj, objanc, 
+	  open_braces += generate_lily_for_obj (gui, &iter, invisibility, curobj, objanc, 
 				 &prevduration, &prevnumdots, &clefname,
 				 &keyname,
 				 &cur_stime1, &cur_stime2);
@@ -1352,24 +1398,25 @@ outputStaff (DenemoGUI *gui, DenemoScore * si, DenemoStaff * curstaffstruct,
 
 	  if( (curobjnode==NULL) || (curobjnode->next==NULL)) {	//at end of measure
 	    GString *endstr = g_string_new("");
-	    if (empty_measure)// measure has nothing to use up the duration, assume whole measure rest
+	    if (empty_measure)// measure has nothing to use up the duration, assume whole measure rest for primary voice, skip for secondaries.
 	      {
-		g_string_append_printf(endstr, "R1*%d/%d ", cur_stime1, cur_stime2);
+		g_string_append_printf(endstr, " %c1*%d/%d ",(curstaffstruct->voicenumber == 1)?'R':'s', cur_stime1, cur_stime2);// either R1 or s1 for whole measure rest or skip
 		gtk_text_buffer_get_iter_at_mark (gui->textbuffer, &iter, curmark);
 		gtk_text_buffer_insert_with_tags_by_name (gui->textbuffer, &iter, endstr->str, -1,invisibility,NULL);
 		g_string_assign(endstr,"");
-		prevduration = 0;
+		prevduration = -1;
 	      }
 	    if(figures->len)
 	      g_string_append(figures, "\n");
 	    if(fakechords->len)
 	      g_string_append(fakechords, "\n");
-	    if(curobj == NULL || curobj->type!=LILYDIRECTIVE) /* if it ends in a lilydirective, the user may want to choose their own
-					       barline style, let them */
+	    
+	    if((curobjnode!=NULL) && ((curobj == NULL) || curobj->type!=LILYDIRECTIVE)) /* if it ends in a lilydirective, the user may want to choose their own barline style, let them */
 	      if (curmeasure->next)
-		g_string_append_printf(endstr, "|\n");
+		g_string_append_printf(endstr, "%|\n");
 	      else
 		g_string_append_printf(endstr, " \\bar \"|.\"\n");
+	    
 	    gtk_text_buffer_get_iter_at_mark (gui->textbuffer, &iter, curmark);
 	    gtk_text_buffer_insert_with_tags_by_name (gui->textbuffer, &iter, endstr->str, -1, INEDITABLE, invisibility,NULL);
 	    //g_string_assign(endstr,"");
@@ -1395,11 +1442,16 @@ outputStaff (DenemoGUI *gui, DenemoScore * si, DenemoStaff * curstaffstruct,
 	  if(curobjnode==NULL || curobjnode->next==NULL)
 	    break;//we want to go through once for empty measures
 	} /* For each object in the measure */
-
-
-
     } /* for each staff */
   
+     
+  for(;open_braces>0;open_braces--)
+    g_string_append_printf(str, "\n} %% missing close brace\n");
+
+
+
+
+
   // str is empty again now FIXME
   g_string_append_printf(str, "}\n");
   g_string_append_printf(str, "%s%sMusicVoice = \\context Voice = %s%s %s {\\%s%sProlog \\%s%s}\n",
@@ -1444,7 +1496,7 @@ outputStaff (DenemoGUI *gui, DenemoScore * si, DenemoStaff * curstaffstruct,
 			"\\set figuredBassAlterationDirection = #1\n"
 			"\\set figuredBassPlusDirection = #1\n"
 			"\\override FiguredBass.BassFigure "
-			"#'font-size = #-2\n",movement, voice);
+			"#'font-size = #-1\n",movement, voice);
 	insert_editable(&curstaffstruct->figures_prolog, temp->str, &iter,  invisibility, gui);
       }
       g_string_printf(temp, "%s \n}\n", figures->str);
@@ -1829,12 +1881,15 @@ output_score_to_buffer (DenemoGUI *gui, gboolean all_movements, gchar * partname
 	  GString *str = g_string_new("");
 	  
 	  gchar *staffprolinsert =  (curstaffstruct->staff_prolog_insert && curstaffstruct->staff_prolog_insert->len)? curstaffstruct->staff_prolog_insert->str:"";
-	  if (curstaffstruct->context & DENEMO_CHOIR_START)
-	    g_string_append_printf(str, "\\new ChoirStaff %s << \n", staffprolinsert);
-	  if (curstaffstruct->context & DENEMO_GROUP_START)
-	    g_string_append_printf(str, "\\new StaffGroup %s << \n", staffprolinsert);
-	  if (curstaffstruct->context & DENEMO_PIANO_START) /* Piano staff cannot start before Group */
-	    g_string_append_printf(str, "\\new PianoStaff %s << \n", staffprolinsert);
+
+	  if(partname==NULL) {//when printing just one part, do not start/stop contexts
+	    if (curstaffstruct->context & DENEMO_CHOIR_START)
+	      g_string_append_printf(str, "\\new ChoirStaff %s << \n", staffprolinsert);
+	    if (curstaffstruct->context & DENEMO_GROUP_START)
+	      g_string_append_printf(str, "\\new StaffGroup %s << \n", staffprolinsert);
+	    if (curstaffstruct->context & DENEMO_PIANO_START) /* Piano staff cannot start before Group */
+	      g_string_append_printf(str, "\\new PianoStaff %s << \n", staffprolinsert);
+	  }
 
 	  if(curstaffstruct->voicenumber == 1)
 	    g_string_append_printf(str, "\\new Staff %s << {\n", staffprolinsert);
@@ -1868,15 +1923,14 @@ output_score_to_buffer (DenemoGUI *gui, gboolean all_movements, gchar * partname
 	    g_string_append_printf(scoreblock, ""TAB""TAB" \\context FiguredBass \\%s%sBassFiguresLine\n", movement_name->str, voice_name->str);
 
 
-
-
+	  if(partname==NULL) {
 	    if(curstaffstruct->context & DENEMO_PIANO_END)
 	      g_string_append_printf(scoreblock, ">>\n\n");
 	    if(curstaffstruct->context & DENEMO_CHOIR_END)
 	      g_string_append_printf(scoreblock, ">>\n\n");
 	    if(curstaffstruct->context & DENEMO_GROUP_END)
 	      g_string_append_printf(scoreblock, ">>\n\n");
-	  
+	  }
 
        
 	}
@@ -1889,8 +1943,12 @@ output_score_to_buffer (DenemoGUI *gui, gboolean all_movements, gchar * partname
      
       g_string_append_printf(scoreblock,
 			     ">>\n>>\n"
-			     ""TAB"\\layout {\n"
-			     ""TAB"}\n");
+			     ""TAB"\\layout {\n");
+			      /* if any layout markup place it in the \layout block */
+			       if(si->headerinfo.layout->len)
+				 g_string_append_printf (scoreblock, ""TAB""TAB"%s\n", si->headerinfo.layout->str);
+
+      g_string_append_printf(scoreblock, ""TAB"}\n");
      
       /* \header block */
       g_string_append_printf(scoreblock, "\\header{\n");
