@@ -435,8 +435,16 @@ midi_end_of_track (FILE * fd)
 void
 midi_timesig (FILE * fd, int upper, int lower)
 {
+  div_t n;	
+  int click = 24;
+
+  if (lower !=0){
+    n = div(4, lower);
+    click = 24 * ((float) n.quot + (float) n.rem);
+  }
+
   midi_4_bytes (fd, 0, 0xff, 0x58, 4);
-  midi_4_bytes (fd, upper, twolog (lower), 24, 8);
+  midi_4_bytes (fd, upper, twolog (lower), click, 8);
 }
 
 /**
@@ -1159,6 +1167,9 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 			     fmt_ticks (ticks_read));
 		  chordval = *(chord *) curobj->object;
 
+		  if(chordval.is_grace)
+		    break;
+
 	  /***********************************
 	   * compute nominal duration of note 
 	   ***********************************/
@@ -1505,7 +1516,6 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
        *******************/
 
 	  measurewidth = bars2ticks (1, timesigupper, timesiglower);
-
 	  if (ticks_at_bar + measurewidth != ticks_read)
 	    {
 	      if ((!measure_is_empty) && curmeasure->next)
@@ -1523,8 +1533,15 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 			   ticks_at_bar + measurewidth - ticks_read,
 			   ticks_at_bar);
 		}
+ 		if(debug){
+		  printf("\nmeasure is empty = %d", measure_is_empty);
+		  printf("\nticks_at_bar(%d) + measurewidth(%d)!= "
+				  "ticks_read(%d)\n",
+				  ticks_at_bar, measurewidth, ticks_read);
+		  printf("\ninternal ticks = %d\n", internaltoticks(0));
+		}
 
-	      ticks_read = ticks_at_bar + internaltoticks (0);
+	      ticks_read = ticks_at_bar + measurewidth;//+ internaltoticks (0);
 	    }
 	  else
 	    {
