@@ -225,17 +225,33 @@ determineDuration (gint duration, gchar ** durationName)
 
 
 /**
- * Output an integer child as a child of the given node.
+ * return a child node of parent, holding the passed name and integer.
  */
 static xmlNodePtr
 newXMLIntChild (xmlNodePtr parent, xmlNsPtr ns, const xmlChar * name,
 		gint content)
 {
-  static gchar sIntBuf[12];	/* enough for -2000000000 + '\0' */
-  sprintf (sIntBuf, "%d", content);
-  return xmlNewChild (parent, ns, name, (xmlChar *) sIntBuf);
+  gchar *integer = g_strdup_printf("%d", content);
+  xmlNodePtr child = xmlNewChild (parent, ns, name, (xmlChar *) integer);
+  g_free(integer);
+  return child;
 }
 
+/**
+ * 
+Set a prop of the parent, holding the passed name and integer.
+ */
+static void
+newXMLIntProp (xmlNodePtr parent,  const xmlChar * name,
+		gint content)
+{
+  gchar *integer = g_strdup_printf("%d", content);
+  xmlSetProp  (parent, name, (xmlChar *) integer);
+  g_free(integer);
+}
+
+
+
 
 /**
  * Return a newly allocated unique XML ID string which must be freed by the
@@ -536,6 +552,12 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
   ns = xmlNewNs (doc->xmlRootNode, (xmlChar *) DENEMO_XML_NAMESPACE, NULL);
   xmlSetProp (scoreElem, (xmlChar *) "version", (xmlChar *) "2.0");
   /* FIXME: Put comment here ("Denemo XML file generated by..."). */
+
+  if(getNumCharsSchemeText()) {
+    gchar *text = getSchemeText();
+    xmlNewChild (scoreElem, ns, "scheme", (xmlChar *)text);
+    g_free(text);
+  }
   /* lilycontrol for the whole musical score */
 
 
@@ -604,6 +626,8 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
 	       (xmlChar *) si->headerinfo.lilypond_before->str);
   xmlNewChild (parentElem, ns, (xmlChar *) "markup_after",
 	       (xmlChar *) si->headerinfo.lilypond_after->str);
+  xmlNewChild (parentElem, ns, (xmlChar *) "layout_markup",
+	       (xmlChar *) si->headerinfo.layout->str);
 
   /* Output each (primary) staff, and store the IDs in a hash table. */
   fraction = 1.0 / (gdouble) g_list_length (si->thescore);
@@ -762,9 +786,16 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
 	      switch (curObj->type)
 		{
 		case CHORD:
+		  {chord *thechord = (chord *) curObj->object;
 		  emptyMeasure = FALSE;
 
-		  /* If this is the start of a beam, output a <beam-start>. */
+		  if(thechord->prefix && thechord->prefix->len)
+		    xmlNewChild (curElem, ns,
+				 (xmlChar *) "prefix",
+					  (xmlChar *)thechord->prefix->str);
+		  
+		  /* FIXME WHY DO WE EXPORT THIS INFO: it is derived from the data
+		     If this is the start of a beam, output a <beam-start>. */
 
 		  if (curObj->isstart_beamgroup && !curObj->isend_beamgroup)
 		    {
@@ -808,6 +839,18 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
 		    newXMLIntChild (parentElem, ns, (xmlChar *) "dots",
 				    ((chord *) curObj->object)->numdots);
 
+		  /* Output the LilyPond */
+#define DO(field)  if (((chord *) curObj->object)->field \
+                   && ((chord *) curObj->object)->field->len)\
+                      xmlNewChild (objElem, ns, (xmlChar *) #field,\
+				     (xmlChar *) ((chord *) curObj->object)->\
+				     field->str);
+		  DO(prefix);
+		  DO(postfix);
+		  DO(display);
+#undef DO		  
+		  if(((chord *) curObj->object)->chordize)
+		    newXMLIntChild (objElem, ns, (xmlChar *) "chordize", TRUE);
 
 		  /*Output Lyric */
 		  if (((chord *) curObj->object)->lyric)
@@ -847,7 +890,7 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
 		//				(GString *) mych->figure);
 		    
 		  /*Output Fakechords*/
-		  if (((chord *) curObj->object)->fakechord)
+		  if (((chord *) curObj->object)->fakechord && ((GString *) ((chord *) curObj->object)->fakechord)->len)
 		    {
 		    	
   			   GString *temp = g_string_new("");
@@ -1104,6 +1147,9 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
 			xmlNewChild (objElem, ns, (xmlChar *) "tie", NULL);
 		    }
 
+ 
+
+
 		  /* Output all the notes, if this isn't a rest. */
 
 		  if (((chord *) curObj->object)->notes != NULL)
@@ -1135,10 +1181,18 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
 			      newXMLNoteHead (curElem, ns,
 					      curNote->noteheadtype);
 			    }
-			  if(curNote->directive && curNote->directive->len)
+			  if(curNote->postfix && curNote->postfix->len)
 			     xmlNewChild (curElem, ns,
-					  (xmlChar *) "directive",
-					  (xmlChar *)curNote->directive->str);
+					  (xmlChar *) "postfix",
+					  (xmlChar *)curNote->postfix->str);
+                          if(curNote->prefix && curNote->prefix->len)
+			     xmlNewChild (curElem, ns,
+					  (xmlChar *) "prefix",
+					  (xmlChar *)curNote->prefix->str);
+                          if(curNote->display && curNote->display->len)
+			     xmlNewChild (curElem, ns,
+					  (xmlChar *) "display",
+					  (xmlChar *)curNote->display->str);
 			}
 		    }
 
@@ -1163,7 +1217,7 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
 			  lastBeamStartXMLID = NULL;
 			}
 		    }
-
+		  }
 		  break;
 
 		case TUPOPEN:
@@ -1296,7 +1350,16 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
 				     directive->str);
 		  xmlSetProp (objElem, (xmlChar *) "locked",
 			      (xmlChar *) (((lilydirective *) curObj->object)->locked?"true":"false"));
+		  if(((lilydirective *) curObj->object)->display && ((lilydirective *) curObj->object)->display->len) {
+		    xmlSetProp (objElem, (xmlChar *) "display",
+			      (xmlChar *) (((lilydirective *) curObj->object)->display->str));
+		    newXMLIntProp (objElem, (xmlChar *) "x",
+				   (((lilydirective *) curObj->object)->x));
+		    newXMLIntProp (objElem, (xmlChar *) "y",
+				   (((lilydirective *) curObj->object)->y));
 
+		  }
+		    
 		  break;
 		case BARLINE:
 		case MEASUREBREAK:
