@@ -37,8 +37,8 @@
 
 
 
-static void
-file_open (DenemoGUI * gui, gboolean template, ImportType type);
+static gboolean
+file_open (DenemoGUI * gui, gboolean template, ImportType type, gchar *filename);
 
 
 
@@ -196,9 +196,11 @@ set_gui_filename (DenemoGUI * gui, gchar * filename)
 #ifdef DEBUG
       g_print ("%s not in history list\n", gui->filename->str);
 #endif
-      if (g_queue_get_length (Denemo.prefs.history) > MAX_HISTORY)
+      g_print("max history now %d\n", Denemo.prefs.maxhistory);
+      if (g_queue_get_length (Denemo.prefs.history) > Denemo.prefs.maxhistory)
 	{
 	  gpointer data = g_queue_pop_head (Denemo.prefs.history);
+	  g_print("losing one history\n");
 	  if (data)
 	    g_free (data);
 	}
@@ -324,10 +326,62 @@ create_filename (const gchar * file_name, gint *format_id)
   return (g_strconcat (file_name, FORMAT_EXTENSION (*format_id), NULL));
 }
 
+/* Save gui in the file in format format_id to the file filename (or gui->filename
+   if filename is NULL)
+   If there is a scheme script, offers to save that with the file.
+ */
+static void save_in_format(gint format_id, DenemoGUI * gui, gchar *filename) {
+  gchar *file = filename? filename:gui->filename->str;
+  switch (format_id)
+    {
+    case DENEMO_FORMAT:
+    case DNM_FORMAT:
+      {
+	// HERE examine Denemo.Script and if present ask it it should be saved with the file, if not delete the script.
+	    
+	if(getNumCharsSchemeText())
+	  if(!confirm("You have a Script defined", "Use this script every time this file is opened?")) {
+	    deleteSchemeText();
+	  }										 
+	exportXML (file, gui, 0, 0);
+	break;
+      };
+    case MUDELA_FORMAT:
+      {
+	gui->si->markstaffnum = 0;
+	exportlilypond (file, gui, TRUE);
+	break;
+      };
+    case PNG_FORMAT:
+      {
+	gui->lilycontrol.excerpt = TRUE;
+	exportlilypond (file, gui, TRUE);
+	break;
+      }
+    case ABC_FORMAT:
+      {
+	exportabc (file, gui, 0, 0);
+	break;
+      };
+    case MIDI_FORMAT:
+      {
+	exportmidi (file, gui->si, 0, 0);
+	break;
+      };
+    case CSOUND_FORMAT:
+      {
+	exportcsound (file, gui->si, 0, 0);
+	break;
+      };
+    default:
+      break;
+    };
+	
+}
+
 /**
  * File save called by fileselsave callback
  * param file_name is full path to file possibly with extension
- * FIXME duplicate code with file_save
  */
 static void
 filesel_save (DenemoGUI * gui, const gchar * file_name, gint format_id, gboolean template)
@@ -350,58 +404,20 @@ filesel_save (DenemoGUI * gui, const gchar * file_name, gint format_id, gboolean
     set_gui_filename (gui, file);
   }
   basename = g_path_get_basename (file);
-  /* we don't want to save scores under ".denemo",
-     ".jtf" and so on. do we? */
-  if (basename[0] != '.')
-    {
-      switch (format_id)
-	{
-	case DENEMO_FORMAT:
-	case DNM_FORMAT:
-	  {
-	    exportXML (file, gui, 0, 0);
-	    break;
-	  };
-	case MUDELA_FORMAT:
-	  {
-	    gui->si->markstaffnum = 0;
-	    exportlilypond (file, gui, TRUE);
-	    break;
-	  };
-	case PNG_FORMAT:
-	  {
-	    gui->lilycontrol.excerpt = TRUE;
-            exportlilypond (file, gui, TRUE);
-	    break;
-	  }
-	case ABC_FORMAT:
-	  {
-	    exportabc (file, gui, 0, 0);
-	    break;
-	  };
-	case MIDI_FORMAT:
-	  {
-	    exportmidi (file, si, 0, 0);
-	    break;
-	  };
-	case CSOUND_FORMAT:
-	  {
-	    exportcsound (file, si, 0, 0);
-	    break;
-	  };
-	default:
-	  break;
-	};
-	
-	/*export parts as lilypond files*/
-	if(Denemo.prefs.saveparts)
-		export_lilypond_parts(file,gui);
-	if(gui->lilysync==gui->changecount)
-	  gui->lilysync = 0;//still in sync
-	gui->changecount = 0;
-	score_status(gui, FALSE);
 
-	si->readonly = FALSE;
+  if (basename[0] != '.') // avoids empty filename
+    {
+      save_in_format(format_id, gui, file);
+      
+      /*export parts as lilypond files*/
+      if(Denemo.prefs.saveparts)
+	export_lilypond_parts(file,gui);
+      if(gui->lilysync==gui->changecount)
+	gui->lilysync = 0;//still in sync
+      gui->changecount = 0;
+      score_status(gui, FALSE);
+      
+      si->readonly = FALSE;
     }
   g_free(basename);
   g_free(file);
@@ -432,9 +448,10 @@ typedef enum {
  * LOCAL whether to use the local templates or systemwide templates or examples
  * does nothing if unable to access templates
  */
-static void
-template_open (DenemoGUI * gui, TemplateType local)
+static gboolean
+template_open (DenemoGUI * gui, TemplateType local, gchar *filename)
 {
+  gboolean ret = FALSE;
   if(local==LOCAL) {
     if(local_template_path==NULL) {
       init_local_path();
@@ -462,9 +479,10 @@ template_open (DenemoGUI * gui, TemplateType local)
     default_template_path = system_template_path;
   }
   if(default_template_path) {
-    file_open (gui, TRUE, REPLACE_SCORE);
+    ret = file_open (gui, TRUE, REPLACE_SCORE, filename);
     gui->filename = g_string_new("");
   }
+  return ret;
 }
 
 
@@ -474,18 +492,19 @@ template_open (DenemoGUI * gui, TemplateType local)
  * Open system template file callback function 
  */
 void
-system_template_open_with_check (GtkAction * action, gpointer param) {
+system_template_open_with_check (GtkAction * action, DenemoScriptParam * param) {
+  GET_1PARAM(action, param, filename);
   DenemoGUI *gui = Denemo.gui;
   if (gui->changecount)
     {
-      if (confirmbox (gui))
+      if (filename==NULL && confirmbox (gui))
 	{
-	  template_open (gui, SYSTEM);
+	  param->status = template_open (gui, SYSTEM, filename);
 	}
     }
   else
     {
-      template_open (gui, SYSTEM);
+      param->status = template_open (gui, SYSTEM, filename);
     }
 }
 
@@ -493,61 +512,70 @@ system_template_open_with_check (GtkAction * action, gpointer param) {
  * Open system template file callback function 
  */
 void
-system_example_open_with_check (GtkAction * action, gpointer param) {
+system_example_open_with_check (GtkAction * action, DenemoScriptParam * param) {
+  GET_1PARAM(action, param, filename);
   DenemoGUI *gui = Denemo.gui;
   if (gui->changecount)
     {
       if (confirmbox (gui))
 	{
-	  template_open (gui, EXAMPLE);
+	  param->status = template_open (gui, EXAMPLE, filename);
 	}
     }
   else
     {
-      template_open (gui, EXAMPLE);
+      param->status = template_open (gui, EXAMPLE, filename);
     }
 }
 /*
  * Open local template file callback function 
  */
 void
-local_template_open_with_check (GtkAction * action, gpointer param) {
+local_template_open_with_check (GtkAction * action, DenemoScriptParam * param) {
+  GET_1PARAM(action, param, filename);
   DenemoGUI *gui = Denemo.gui;
   if (gui->changecount)
     {
-      if (confirmbox (gui))
+      if (filename==NULL && confirmbox (gui))
 	{
-	  template_open (gui, LOCAL);
+	  param->status = template_open (gui, LOCAL, filename);
 	}
     }
   else
     {
-      template_open (gui, LOCAL);
+      param->status = template_open (gui, LOCAL, filename);
     }
 }
 
 
 /**
- * Wrapper function for opening a file
- * checks to see if current score has changed and prompts user to save 
+ * Wrapper function for opening a file, d-Open
+ * if no param checks to see if current score has changed and prompts user to save 
  * otherwise opens the file
  */
 void
-file_open_with_check (GtkAction * action, gpointer param)
+file_open_with_check (GtkAction * action, DenemoScriptParam * param)
 {
+  GET_1PARAM(action, param, filename);
+  if(query){
+    param->status = (Denemo.gui->filename!=NULL) && Denemo.gui->filename->len;
+    if(param->status)
+      g_string_assign(param->string, Denemo.gui->filename->str);
+    return;
+  }
   DenemoGUI *gui = Denemo.gui;
   if (gui->changecount)
     {
       if (confirmbox (gui))
 	{
 	  deletescore (NULL, gui);
-	  file_open (gui, FALSE, REPLACE_SCORE);
+	  param->status = file_open (gui, FALSE, REPLACE_SCORE, filename);
 	}
     }
   else
     {
       deletescore (NULL, gui);
-      file_open (gui, FALSE, REPLACE_SCORE);
+      param->status = file_open (gui, FALSE, REPLACE_SCORE, filename);
     }
 }
 
@@ -557,11 +585,12 @@ file_open_with_check (GtkAction * action, gpointer param)
  * 
  */
 void
-file_add_movements(GtkAction * action, gpointer param){
+file_add_movements(GtkAction * action,  DenemoScriptParam * param){
   DenemoGUI *gui = Denemo.gui;
-  if(!confirm_insertstaff_custom_scoreblock(gui))
+  GET_1PARAM(action, param, filename);
+  if(filename==NULL && !confirm_insertstaff_custom_scoreblock(gui))
     return;
-  file_open(gui, FALSE, ADD_MOVEMENTS);
+  param->status = file_open(gui, FALSE, ADD_MOVEMENTS, filename);
   score_status(gui, TRUE);
 }
 /**
@@ -569,11 +598,12 @@ file_add_movements(GtkAction * action, gpointer param){
  * 
  */
 void
-file_add_staffs(GtkAction * action, gpointer param){
+file_add_staffs(GtkAction * action,  DenemoScriptParam * param){
+  GET_1PARAM(action, param, filename);
   DenemoGUI *gui = Denemo.gui;
-  if(!confirm_insertstaff_custom_scoreblock(gui))
+  if(filename==NULL && !confirm_insertstaff_custom_scoreblock(gui))
     return;
-  file_open(gui, FALSE, ADD_STAFFS);
+   param->status = file_open(gui, FALSE, ADD_STAFFS, filename);
   score_status(gui, TRUE);
 }
 
@@ -602,11 +632,15 @@ static void  set_current_folder(GtkWidget *file_selection, DenemoGUI *gui, gbool
 }
 /**
  * File open dialog - opened where appropriate 
- *
+ * return TRUE on success FALSE on failure.
  */
-void
-file_open (DenemoGUI * gui, gboolean template, ImportType type)
+static gboolean
+file_open (DenemoGUI * gui, gboolean template, ImportType type, gchar *filename)
 {
+  gboolean ret = FALSE;
+  if(filename)
+    return (0==open_for_real(filename, gui, template, type));
+
   GtkWidget *file_selection;
   GtkFileFilter *filter;
 
@@ -642,17 +676,17 @@ file_open (DenemoGUI * gui, gboolean template, ImportType type)
   gtk_widget_show_all (file_selection);
   if (gtk_dialog_run (GTK_DIALOG (file_selection)) == GTK_RESPONSE_ACCEPT)
     {
-      gchar *filename =
+      gchar *name =
 	gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_selection));
-      if(open_for_real (filename, gui, template, type)) {
-	gchar *warning = g_strdup_printf("Load of file %s failed", filename);
+      if((ret=open_for_real (name, gui, template, type))) {
+	gchar *warning = g_strdup_printf("Load of file %s failed", name);
 	warningdialog(warning);
 	g_free(warning);
       }
-      g_free (filename);
+      g_free (name);
     }
   gtk_widget_destroy (file_selection);
-
+  return ret;
 }
 
 /**
@@ -694,8 +728,8 @@ file_savewrapper (GtkAction * action, gpointer param)
 }
 
 /**
- * Filters the filename based on its extension and calls
- * the relevant export function
+ * if gui->filename exists saves gui to the filename  based on its extension
+ * otherwise call saveas routine
  */
 void
 file_save (GtkWidget * widget, DenemoGUI * gui)
@@ -706,6 +740,8 @@ file_save (GtkWidget * widget, DenemoGUI * gui)
     /* No filename's been given or is opened from template */
     file_saveas (gui, FALSE);
   else
+    save_in_format(guess_file_format (gui->filename->str), gui, NULL);
+#if 0
     switch (guess_file_format (gui->filename->str))
       {
       case DENEMO_FORMAT:
@@ -742,6 +778,7 @@ file_save (GtkWidget * widget, DenemoGUI * gui)
 	  break;
 	};
       };
+#endif
    /*Save parts as lilypond files*/   
    if(Denemo.prefs.saveparts)
 	export_lilypond_parts(gui->filename->str,gui);
