@@ -16,8 +16,10 @@
 #include "tupletops.h"
 #include "xmldefs.h"
 #include "articulations.h"
+#include "view.h"
 #include <string.h>
-#include <glib.h>
+
+
 /* libxml includes: for libxml2 this should be <libxml.h> */
 #include <libxml/parser.h>
 #include <libxml/tree.h>
@@ -634,10 +636,10 @@ parseFakechord (xmlNodePtr fakechordElem, DenemoObject * curobj)
   gchar *fakechord = (gchar *) xmlNodeListGetString (fakechordElem->doc,
 						  fakechordElem->xmlChildrenNode,
 						  1);
-  
-  ((chord *) curobj->object)->is_fakechord = TRUE; 
-  separate_fakechord_elements (fakechord, curobj); 
-  
+  if(fakechord) {
+    ((chord *) curobj->object)->is_fakechord = TRUE; 
+    separate_fakechord_elements (fakechord, curobj); 
+  }
   //((chord *) curobj->object)->fakechord = g_string_new (fakechord);
 
   g_free (fakechord);
@@ -656,7 +658,7 @@ parseNote (xmlNodePtr noteElem, xmlNsPtr ns,
   gint middleCOffset = 0, accidental = 0,
     noteHeadType = DENEMO_NORMAL_NOTEHEAD;
   gboolean showAccidental = FALSE;
-  gchar *accidentalName, *showAccidentalProp, *noteHeadName, *directive = NULL;
+  gchar *accidentalName, *showAccidentalProp, *noteHeadName, *postfix = NULL, *prefix = NULL, *display = NULL;
 
   FOREACH_CHILD_ELEM (childElem, noteElem)
   {
@@ -716,9 +718,19 @@ parseNote (xmlNodePtr noteElem, xmlNsPtr ns,
 		g_free (showAccidentalProp);
 	      }
 	  }
-	else if (ELEM_NAME_EQ (childElem, "directive"))
+	else if (ELEM_NAME_EQ (childElem, "postfix"))
 	  {
-	    directive = (gchar *) xmlNodeListGetString
+	    postfix = (gchar *) xmlNodeListGetString
+	      (childElem->doc, childElem->xmlChildrenNode, 1);	    
+	  }
+	else if (ELEM_NAME_EQ (childElem, "prefix"))
+	  {
+	    prefix = (gchar *) xmlNodeListGetString
+	      (childElem->doc, childElem->xmlChildrenNode, 1);	    
+	  }
+	else if (ELEM_NAME_EQ (childElem, "display"))
+	  {
+	    display = (gchar *) xmlNodeListGetString
 	      (childElem->doc, childElem->xmlChildrenNode, 1);	    
 	  }
 	else if (ELEM_NAME_EQ (childElem, "note-head"))
@@ -758,8 +770,13 @@ parseNote (xmlNodePtr noteElem, xmlNsPtr ns,
   /* Now actually construct the note object. */
 
   note *newnote = addtone (chordObj, middleCOffset, accidental, currentClef);
-  if(newnote && directive)
-    newnote->directive = g_string_new(directive);
+  if(newnote && postfix)
+    newnote->postfix = g_string_new(postfix);
+  if(newnote && prefix)
+    newnote->prefix = g_string_new(prefix);
+  if(newnote && display)
+    newnote->display = g_string_new(display);
+
   if (noteHeadType != DENEMO_NORMAL_NOTEHEAD)
     {
       /* FIXME: Is this the right note in the chord? */
@@ -1308,6 +1325,33 @@ parseChord (xmlNodePtr chordElem, xmlNsPtr ns,
 	    parseFakechord (childElem, chordObj);
 	    si->has_fakechords = (gpointer) TRUE;
 	  }
+	else if (ELEM_NAME_EQ (childElem, "prefix"))
+	  {
+	    ((chord *) chordObj->object)->prefix = g_string_new(xmlNodeListGetString (childElem->doc,
+						  childElem->xmlChildrenNode,
+						  1));
+	  }
+	else if (ELEM_NAME_EQ (childElem, "postfix"))
+	  {
+	    ((chord *) chordObj->object)->postfix = g_string_new(xmlNodeListGetString (childElem->doc,
+						  childElem->xmlChildrenNode,
+						  1));
+	  }
+	else if (ELEM_NAME_EQ (childElem, "display"))
+	  {
+	    ((chord *) chordObj->object)->display = g_string_new(xmlNodeListGetString (childElem->doc,
+						  childElem->xmlChildrenNode,
+						  1));
+	  }
+
+	else if (ELEM_NAME_EQ (childElem, "chordize"))
+	  {
+	    ((chord *) chordObj->object)->chordize = TRUE;
+	  }
+
+
+
+
 
 	else
 	  {
@@ -1336,10 +1380,27 @@ parseLilyDir (xmlNodePtr LilyDirectiveElem, xmlNsPtr ns, DenemoScore *si)
 						     xmlChildrenNode,
 						     1);
   gchar *locked = (gchar *) xmlGetProp (LilyDirectiveElem, (xmlChar *) "locked");
+  gchar *display = (gchar *) xmlGetProp (LilyDirectiveElem, (xmlChar *) "display");
+  gchar *x = (gchar *) xmlGetProp (LilyDirectiveElem, (xmlChar *) "x");
+  gchar *y = (gchar *) xmlGetProp (LilyDirectiveElem, (xmlChar *) "y");
+
+
   DenemoObject *curobj = lily_directive_new (directive);
   if(locked)
     ((lilydirective*)curobj->object)->locked = !strcmp (locked, "true");
   g_free(locked);
+  if(display)
+    ((lilydirective*)curobj->object)->display = g_string_new(display);
+  g_free(display);
+  if(x)
+    ((lilydirective*)curobj->object)->x = atoi(x);
+  g_free(x);
+  if(y)
+    ((lilydirective*)curobj->object)->y = atoi(y);
+  g_free(y);
+
+
+
   return curobj;
 }
 
@@ -1583,7 +1644,7 @@ parseScoreInfo (xmlNodePtr scoreInfoElem, xmlNsPtr ns, DenemoScore * si)
   gint bpm;
   gchar *title, *subtitle, *composer, *poet, *meter, *arranger, *opus;
   gchar *instrument, *dedication, *piece, *head, *copyright, *footer,
-    *tagline, *mvmnt_header, *markup_before, *markup_after;
+    *tagline, *mvmnt_header, *markup_before, *markup_after, *layout_markup;
 
   FOREACH_CHILD_ELEM (childElem, scoreInfoElem)
   {
@@ -1811,6 +1872,17 @@ parseScoreInfo (xmlNodePtr scoreInfoElem, xmlNsPtr ns, DenemoScore * si)
 		g_free (markup_after);
 	      }
 	  }
+	else if (ELEM_NAME_EQ (childElem, "layout_markup"))
+	  {
+	    layout_markup = (gchar *) xmlNodeListGetString (childElem->doc,
+						      childElem->
+						      xmlChildrenNode, 1);
+	    if (layout_markup != NULL)
+	      {
+		g_string_assign (si->headerinfo.layout, layout_markup);
+		g_free (layout_markup);
+	      }
+	  }
 
 
 
@@ -2631,8 +2703,19 @@ importXML (gchar * filename, DenemoGUI *gui, ImportType type)
       break;
     case REPLACE_SCORE:
       free_gui(gui);
+      deleteSchemeText();
       /* this is dependent on the order of elements, which is not strictly correct */
       FOREACH_CHILD_ELEM(childElem, rootElem){
+
+	if (ELEM_NAME_EQ (childElem, "scheme")) {
+	  gchar *tmp = (gchar *) xmlNodeListGetString (childElem->doc,
+						  childElem->
+						  xmlChildrenNode, 1);
+	  if (tmp != NULL) {
+	    appendSchemeText(tmp);
+	    g_free (tmp);
+	  }
+	} else
 	if (ELEM_NAME_EQ (childElem, "lilycontrol")){
 	  parseSetupInfo(childElem, ns, gui);
 	} else
@@ -2662,6 +2745,9 @@ importXML (gchar * filename, DenemoGUI *gui, ImportType type)
 	      ret |=  parseMovement(childElem, ns, gui, type);
 	    }
       }
+
+      if(getNumCharsSchemeText())
+	  executeScript(); 
       break;
     default:
       warningdialog("Erroneous call");
