@@ -32,7 +32,10 @@ typedef struct _keyboard_dialog_data
 
 guint
 dnm_sanitize_key_event(GdkEventKey *event);
-
+guint
+dnm_hyper_sanitize_key_event(GdkEventKey *event);
+guint
+dnm_meta_sanitize_key_state(GdkEventKey *event);
 gboolean
 isModifier(GdkEventKey *event);
 
@@ -87,20 +90,18 @@ keymap_foreach_command_binding (keymap *the_keymap, guint command_idx,
 
 //-1 if the binding is not found
 gint
-lookup_keybinding (keymap *the_keymap, gint keyval, GdkModifierType state);
+lookup_command_for_keybinding (keymap *the_keymap, gint keyval, GdkModifierType state);
+
+//-1 if the binding is not found
+gint lookup_command_for_keyevent(GdkEventKey * event);
 
 //-1 if the binding is not found
 gint
-lookup_keybinding_from_string (keymap *the_keymap, const gchar *binding_name);
+lookup_command_for_keybinding_name (keymap *the_keymap, const gchar *binding_name);
 
-//GList*
-//lookup_keybindings_from_name(keymap* keymap, const gchar* name);
-
-//GList*
-//lookup_keybindings_by_idx(keymap* keymap, guint idx);
 
 gint 
-lookup_index_from_name (keymap * keymap, const gchar *command_name);
+lookup_command_from_name (keymap * keymap, const gchar *command_name);
 
 const gchar *
 lookup_name_from_idx(keymap *keymap, guint command_idx);
@@ -111,41 +112,44 @@ lookup_label_from_idx(keymap *keymap, guint command_idx);
 const gchar *
 lookup_tooltip_from_idx(keymap *keymap, guint command_idx);
 
+gboolean
+lookup_hidden_from_idx (keymap * keymap, guint command_idx);
+
 gpointer
 lookup_callback_from_idx (keymap * keymap, guint command_idx);
 
-//const gchar *
-//lookup_label_from_name(keymap *keymap, const gchar *command_name);
-
 void
 remove_keybinding (keymap *the_keymap, gint keyval, GdkModifierType state);
 
 void
-remove_keybinding_from_string (keymap *the_keymap, const gchar *binding);
+remove_keybinding_from_name (keymap *the_keymap, const gchar *binding);
 
 typedef enum {
 	POS_FIRST = 0,
 	POS_LAST
-} KbdPosition;
+} ListPosition;
 
 gint
-add_keybinding_from_idx (keymap * the_keymap, gint keyval,
-        GdkModifierType state, guint command_idx, KbdPosition pos);
+add_keybinding_to_idx (keymap * the_keymap, gint keyval,
+        GdkModifierType state, guint command_idx, ListPosition pos);
 gint
-add_keybinding_from_name (keymap *the_keymap, gint keyval,
-        GdkModifierType state, const gchar *command_name, KbdPosition pos);
-
+add_keybinding_to_named_command (keymap *the_keymap, gint keyval,
+        GdkModifierType state, const gchar *command_name, ListPosition pos);
+gint
+add_named_binding_to_idx (keymap * the_keymap, gchar *kb_name,
+			  guint command_idx, ListPosition pos);
 gint
 keymap_update_accel(keymap *the_keymap, GtkAction *action, guint keyval,
 		GdkModifierType modifiers);
 
 gint
-keymap_accel_quick_edit_snooper(GtkWidget *grab_widget, GdkEventKey *event,
-		gpointer func_data);
+keymap_accel_quick_edit_snooper(GtkWidget *grab_widget, GdkEventKey *event);
 
 GtkAction *
-action_of_name(keymap *the_keymap, gchar *command_name);
-
+lookup_action_from_name(gchar *command_name);
+#define action_of_name(a,b) lookup_action_from_name(b)
+const GtkAction *
+lookup_action_from_idx (keymap * keymap, guint command_idx);
 gboolean
 execute_callback_from_idx(keymap *the_keymap, guint command_idx);
 gboolean
@@ -163,14 +167,14 @@ load_keymap_dialog (GtkWidget *Widget);
 
 
 void
-load_default_keymap_file (keymap *the_keymap);
+load_default_keymap_file (void);
 
 void
 load_system_keymap_dialog (GtkWidget * widget);
 void
 save_keymap_dialog (GtkWidget *widget);
 void
-load_keymap_dialog_location (GtkWidget * widget, keymap * the_keymap, gchar *location);
+load_keymap_dialog_location (GtkWidget * widget, gchar *location);
 void
 save_default_keymap_file_wrapper (GtkAction *action, gpointer param);
 
