@@ -104,8 +104,8 @@ get_state (gchar * key)
 /* add ui elements for menupath if missing */
 static gint
 instantiate_menus(gchar *menupath) {
-  /* just add last one for the moment */
   //g_print("Instantiate menus for %s\n", menupath);
+ 
   gchar *up1 = g_path_get_dirname(menupath);
   gchar *name=g_path_get_basename(menupath);
   GtkWidget *widget = gtk_ui_manager_get_widget(Denemo.ui_manager, up1);
@@ -123,6 +123,26 @@ instantiate_menus(gchar *menupath) {
   // widget = gtk_ui_manager_get_widget(Denemo.ui_manager, menupath);
   //show_type (widget, "for menupath widget is ");
 }
+
+void set_visibility_for_action(GtkAction *action, gboolean visible) {
+  if(GTK_IS_ACTION(action)) {
+    GSList *h = gtk_action_get_proxies (action);
+    for(;h;h=h->next) {
+      if(visible)
+	gtk_widget_show(h->data);
+      else
+	gtk_widget_hide(h->data);
+    }
+    g_object_set_data(G_OBJECT(action), "hidden", (gpointer)!visible);
+  }
+
+}
+
+static void hide_action_of_name(gchar *name){
+  GtkAction *action = lookup_action_from_name (name);
+  set_visibility_for_action(action, FALSE);
+}
+
 static void
 parseScripts (xmlDocPtr doc, xmlNodePtr cur, keymap * the_keymap, gchar *fallback, gboolean merge)
 {
@@ -134,13 +154,13 @@ parseScripts (xmlDocPtr doc, xmlNodePtr cur, keymap * the_keymap, gchar *fallbac
   gboolean is_script = FALSE;
   GdkModifierType state = 0;
   DenemoGUI *gui = Denemo.gui;
+  gboolean hidden=FALSE;
   for ( ;cur; cur = cur->next)
     {
-
       if (0 == xmlStrcmp (cur->name, (const xmlChar *) "action")) {
 	if (cur->xmlChildrenNode == NULL)
 	  {
-	    g_warning ("Empty children node found in keymap file\n");
+	    g_warning ("Empty action node found in keymap file\n");
 	  }
 	else
 	  {
@@ -153,6 +173,8 @@ parseScripts (xmlDocPtr doc, xmlNodePtr cur, keymap * the_keymap, gchar *fallbac
       } else if (0 == xmlStrcmp (cur->name, (const xmlChar *) "scheme")) {
 	scheme = 
 	  xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
+#if 0
+	/* this puts the loaded script into the scheme view window - useful for development work */
 	if(merge && scheme) {
 	  GtkTextIter enditer;
 	  GtkTextBuffer *buffer = gtk_text_view_get_buffer((GtkTextView*)(Denemo.ScriptView));
@@ -160,9 +182,13 @@ parseScripts (xmlDocPtr doc, xmlNodePtr cur, keymap * the_keymap, gchar *fallbac
 	  gchar *text = g_strdup_printf(";;;Scheme code for command %s\n%s",name,scheme);
 	  gtk_text_buffer_insert(buffer, &enditer, text, -1);
 	  g_free(text); 
-	}  
+	} 
+#endif 
 	is_script = TRUE;
+      }    else if (0 == xmlStrcmp (cur->name, (const xmlChar *) "hidden")) {
+	hidden = TRUE; 
       }
+
       else if (0 == xmlStrcmp (cur->name, (const xmlChar *) "menupath")) {
 	menupath = 
 	  xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
@@ -179,58 +205,81 @@ parseScripts (xmlDocPtr doc, xmlNodePtr cur, keymap * the_keymap, gchar *fallbac
 	if(is_script) {
 	  name = name?name:(xmlChar*)"NoName";
 	  label = label?label:(xmlChar*)"No label";
-	  scheme = scheme?scheme:(xmlChar*)";;empty script\n";
+	  scheme = scheme?scheme:(xmlChar*)"";
 	  tooltip = tooltip?tooltip:(xmlChar*)"No indication what this done beyond the name and label :(";
-	  
-	  
-	  //FIXME duplicate code with view.c *************
-	  GtkAction *action = gtk_action_new(name,label,tooltip,NULL);
-	  GtkActionGroup *action_group;
-	  GList *groups = gtk_ui_manager_get_action_groups (Denemo.ui_manager);
-	  action_group = Denemo.action_group; 
-	  gtk_action_group_add_action(action_group, action);
+	  GtkAction *action;
+	  gboolean new_command = FALSE;
+	  action = lookup_action_from_name(name);	     
+	  if(action==NULL) {
+	    new_command = TRUE;
+	    action = gtk_action_new(name,label,tooltip,NULL);
+	    //g_print("New action %s\n", name);
+	    if(hidden)
+	      g_object_set_data(action, "hidden", TRUE);
 
+	    register_command(Denemo.map, action, name, label, tooltip, activate_script);
+	    GtkActionGroup *action_group;
+	    GList *groups = gtk_ui_manager_get_action_groups (Denemo.ui_manager);
+	    action_group = Denemo.action_group; 
+	    gtk_action_group_add_action(action_group, action);
+	    /* create a scheme function to call this script */
+	    create_scheme_function_for_script(name);
+	  } 
 	  if(menupath) {
-	    GList *g;
-	    for(g=menupaths;g;g=g->next){
-	      menupath = g->data;
-	      menupath = menupath?menupath:(xmlChar*)"/MainMenu/Other";
+	      GList *g;
+	      for(g=menupaths;g;g=g->next){
+		menupath = g->data;
+		menupath = menupath?menupath:(xmlChar*)"/MainMenu/Other";
+		GtkWidget *widget = gtk_ui_manager_get_widget(Denemo.ui_manager, menupath);
+		if(widget==NULL) {
+		  instantiate_menus(menupath);
+		}
+		  gtk_ui_manager_add_ui(Denemo.ui_manager,gtk_ui_manager_new_merge_id(Denemo.ui_manager), 
+				      menupath,
+				      name, name, GTK_UI_MANAGER_AUTO, FALSE);
+	      }
+	  } else {
+	    if(fallback) {/* no path given, use fallback */
+	      menupath = fallback;
 	      GtkWidget *widget = gtk_ui_manager_get_widget(Denemo.ui_manager, menupath);
-	      if(widget==NULL)
+	      if(widget==NULL) {
 		instantiate_menus(menupath);
-	      gtk_ui_manager_add_ui(Denemo.ui_manager,gtk_ui_manager_new_merge_id(Denemo.ui_manager), 
-				    menupath,
-				    name, name, GTK_UI_MANAGER_AUTO, FALSE);
+	      }
+		gtk_ui_manager_add_ui(Denemo.ui_manager,gtk_ui_manager_new_merge_id(Denemo.ui_manager), 
+				      menupath,
+				      name, name, GTK_UI_MANAGER_AUTO, FALSE);
 	    }
-
-	  } else if(fallback) {/* no path given, use fallback */
-	    menupath = fallback;
-	    GtkWidget *widget = gtk_ui_manager_get_widget(Denemo.ui_manager, menupath);
-	    if(widget==NULL)
-	      instantiate_menus(menupath);
-	    gtk_ui_manager_add_ui(Denemo.ui_manager,gtk_ui_manager_new_merge_id(Denemo.ui_manager), 
-				  menupath,
-				  name, name, GTK_UI_MANAGER_AUTO, FALSE);
-	    
 	  }
-	  g_object_set_data(G_OBJECT(action), "scheme", scheme);
-	  g_object_set_data(G_OBJECT(action), "menupath", menupath);
-	  g_signal_connect (G_OBJECT (action), "activate",
-			    G_CALLBACK (activate_script), gui);
-	  if(merge) {
+	  if(merge && new_command) {
 	    gchar *msg = g_strdup_printf("Installed a command in the menu system\nat %s\n", menupath);
 	    infodialog(msg);
 	    g_free(msg);
 	  }
-	  //g_print("registering %s\n", name);
-	  register_command(Denemo.commands, action, name, label, tooltip, activate_script);
-	  //end duplicate code **************
-	  
-	}// is_script
+	  /*FIXME free?	  gchar *old_scheme = (gchar *)g_object_get_data(G_OBJECT(action), "scheme");*/
+	  //g_print("Setting scheme %s\n", scheme);
+	  g_object_set_data(G_OBJECT(action), "scheme", scheme);
+	  g_object_set_data(G_OBJECT(action), "menupath", menupath);
+	  if(new_command) {
+	  g_signal_connect (G_OBJECT (action), "activate",
+			    G_CALLBACK (activate_script), gui);
+	  //g_print("Signal activate is set on action %p %s scheme is %s\n", action, name, scheme);
+	  }// else
+	  //g_print("scheme now %s", scheme);
+	  // Note the script should *not* be in Default.cmdset
+	  // to delay loading it, but we should set the signal initally and we should not repeat setting the signal later.
+	  // the signal does not specify which script will be run, that is decided lazily, when the action is invoked for the first time
+
+	} // is_script
 	// we are not as yet re-writing tooltips etc on builtin commands
+#if 1
+	else if(hidden) {
+	  hide_action_of_name(name); hidden = FALSE;
+	  g_print("Hiding Builtin %s\n", name);
+	}
+#endif
       }// tooltip found, assumed last field
     } // for all nodes
-  //alphabeticalize_commands(Denemo.commands);
+  //alphabeticalize_commands(Denemo.map);
 }/* end of parseScripts */
 
 
@@ -264,11 +313,11 @@ parseBindings (xmlDocPtr doc, xmlNodePtr cur, keymap * the_keymap)
 	    }
 	} else if (0 == xmlStrcmp (cur->name, (const xmlChar *) "bind"))
 	{
-	  command_number = lookup_index_from_name (the_keymap, (gchar *) name);
+	  command_number = lookup_command_from_name (the_keymap, (gchar *) name);
 	  //g_print("Found bind node for action %s %d\n", name, command_number);
 	  if (cur->xmlChildrenNode == NULL)
             {
-              g_warning ("Empty children node found in keymap file\n");
+              g_warning ("Empty <bind><\\bind> found in commandset file\n");
             }
 	  else
 	    {
@@ -277,29 +326,77 @@ parseBindings (xmlDocPtr doc, xmlNodePtr cur, keymap * the_keymap)
 	      if (tmp)
 		{
 		  gchar *gtk_binding = translate_binding_dnm_to_gtk((gchar *) tmp);
-          if (gtk_binding) {
-            dnm_accelerator_parse(gtk_binding, &keyval, &state);
+		  //g_print("gtk_binding is %s\n", gtk_binding);
+		  if (gtk_binding) {
+		    dnm_accelerator_parse(gtk_binding, &keyval, &state);
 #ifdef DEBUG
 		    g_print ("binding %s, keyval %d, state %d, Command Number %d\n",
-                  gtk_binding, keyval, state, command_number);
+			     gtk_binding, keyval, state, command_number);
 #endif
-		    if (command_number != -1)
-		        add_keybinding_from_idx (the_keymap, keyval, state,
-						 command_number, POS_LAST);
-//FIXME PROBLEM, there are no proxies for the MyName command, even though we have just created it...
-		  
+		    
+		    if (command_number != -1){
+		      if(keyval) 
+			add_keybinding_to_idx (the_keymap, keyval, state,
+					       command_number, POS_LAST);
+		      else
+			add_named_binding_to_idx (the_keymap, tmp, command_number, POS_LAST);
+		    }
+		    //FIXME PROBLEM, there are no proxies for the MyName command, even though we have just created it...
+		    
 		    g_free (gtk_binding);
-          } else {
-              g_warning("Could not parse binding", (gchar *) tmp);
-          }
+		  } else {
+		    g_print("No gtk equivalent for shortcut %s\n", tmp);
+		  }
 		  xmlFree (tmp);
 		}
 	    }
-	}
+	}  
       cur = cur->next;
     }
 } // parseBindings
 
+static void
+parseCursorBinding (xmlDocPtr doc, xmlNodePtr cur) {
+  gint state, cursor_num;
+  xmlChar *tmp;
+  for (cur = cur->xmlChildrenNode;cur != NULL; cur = cur->next)
+    {
+      if (0 == xmlStrcmp (cur->name, (const xmlChar *) "state")) {
+	tmp = 
+	  xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
+	      if (tmp)
+		{
+		  sscanf(tmp, "%x",&state);// = atoi(tmp);
+		  xmlFree (tmp);
+		}
+	     
+      } else if (0 == xmlStrcmp (cur->name, (const xmlChar *) "cursor"))
+	{
+	tmp = 
+	  xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
+	      if (tmp)
+		{
+		  cursor_num = atoi(tmp);
+		  xmlFree (tmp);
+		}
+	      assign_cursor(state, cursor_num);
+	      // g_print("type is %s\n",g_type_name(G_TYPE_FROM_INSTANCE(Denemo.window->window)));
+	      // set_cursor_for(state);
+	}
+    }
+}
+
+
+static void
+parseCursors (xmlDocPtr doc, xmlNodePtr cur) {
+  xmlChar *tmp;
+  for (cur = cur->xmlChildrenNode;cur != NULL; cur = cur->next)
+    {
+      if (0 == xmlStrcmp (cur->name, (const xmlChar *) "cursor-binding")) {
+	parseCursorBinding(doc, cur);
+      }
+    }
+}
 
 
 
@@ -314,48 +411,69 @@ parseCommands (xmlDocPtr doc, xmlNodePtr cur, keymap * the_keymap, gchar *menupa
       if ((0 == xmlStrcmp (ncur->name, (const xmlChar *) "row")))
 	{
 	  i?parseBindings (doc, ncur, the_keymap):parseScripts (doc, ncur, the_keymap, menupath, merge);
-	}
+	} else 
+	  if (i && (0 == xmlStrcmp (ncur->name, (const xmlChar *) "cursors")))
+	    {
+	      parseCursors(doc, ncur);
+
+	    }
     }
 }
 
+
 static void
 parseKeymap (xmlDocPtr doc, xmlNodePtr cur, keymap * the_keymap, gchar *menupath, gboolean merge)
 {
-  cur = cur->xmlChildrenNode;
-
-  while (cur != NULL)
+  for (cur = cur->xmlChildrenNode; cur != NULL;  cur = cur->next)
     {
       if (0 == xmlStrcmp (cur->name, (const xmlChar *) "map"))
 	{
 	  parseCommands (doc, cur, the_keymap, menupath, merge);
 	}
-      cur = cur->next;
     }
-
 }
 
 /* if filename ends in /menus/.... hierarchy extract and return the tail
    below menus/
 */
 gchar *extract_menupath(gchar *filename) {
+  gchar *head = g_strdup_printf("%c%s", G_DIR_SEPARATOR, "menus");
   GString *str = g_string_new("");
-  gchar *base = g_strrstr(filename, "/menus");
+  gchar *base = g_strrstr(filename, head);
   if(base) {
-    base += strlen("/menus");
+    base += strlen(head);
     base = g_path_get_dirname(base);
+    gchar *c;
+    for(c = base; c && *c;c++)
+      if(*c==G_DIR_SEPARATOR)
+	*c = '/';
     //g_print("got base as %s\n", base);
   }
+  g_free(head);
+
+
+ 
+
+
+
   return base;
 }
 
 
 
-/* returns 0 on success
+/* 
+ * load a command from filename. 
+ * If an action exists but has an empty script load the scheme script,
+ if action is a new action leave script as empty string for loading on demand.
+ * Create a widget for the action in a menupath found in filename or failing that deduced from
+ * the path to filename itself (starting from actions/menus).
+ * returns 0 on success
  * negative on failure
  */
 gint
-load_xml_keymap (gchar * filename, keymap * the_keymap)
+load_xml_keymap (gchar * filename)
 {
+  keymap *the_keymap = Denemo.map; 
   gint ret = -1;
   gboolean merge = FALSE;//Whether to replace the keymap or merge with it
   xmlDocPtr doc;
@@ -404,20 +522,20 @@ load_xml_keymap (gchar * filename, keymap * the_keymap)
 	  )
 	{
 	  if(!merge) {
-	    //g_print("Losing command set %p\n", Denemo.commands);
+	    //g_print("Losing command set %p\n", Denemo.map);
 	    init_keymap(); 
-	    // g_print("Starting with a clean command set %p\n", Denemo.commands);
+	    // g_print("Starting with a clean command set %p\n", Denemo.map);
 	  }
-	  parseKeymap (doc, rootElem, Denemo.commands, menupath, merge);
+	  parseKeymap (doc, rootElem, Denemo.map, menupath, merge);
 	  if(merge) {
 	    if(Denemo.last_merged_command)
 	      g_free(Denemo.last_merged_command);
 	    Denemo.last_merged_command = g_strdup(filename);
-	    Denemo.accelerator_status = TRUE;
+	    //Denemo.accelerator_status = TRUE;
 	  }
 	  //if(!merge)
-	  //  alphabeticalize_commands(Denemo.commands);
-	  update_all_labels(Denemo.commands);
+	  //  alphabeticalize_commands(Denemo.map);
+	  update_all_labels(Denemo.map);
 	  ret = 0;
 	}
       rootElem = rootElem->next;
@@ -440,9 +558,21 @@ write_xml_keybinding_info (gchar *kb_name, xmlNodePtr node)
 
 }
 
+static void
+output_pointer_shortcut(gint *state, GdkCursor *cursor, xmlNodePtr parent){
+  gchar *statestr = g_strdup_printf("%x", *state);
+  gint cursor_num = cursor->type;
+  gchar *numstr = g_strdup_printf("%d", cursor_num);
+  xmlNodePtr child = xmlNewTextChild (parent, NULL, (xmlChar *) "cursor-binding", NULL);
+  xmlNewChild (child, NULL, "state", statestr);
+  xmlNewChild (child, NULL, "cursor", numstr);
+  g_free(statestr);
+  g_free(numstr);
+}
 gint
-save_xml_keymap (gchar * filename, keymap * the_keymap)
+save_xml_keymap (gchar * filename)
 {
+  keymap *the_keymap = Denemo.map;
   gint i, ret = -1;
   xmlDocPtr doc;
   //xmlNsPtr ns;
@@ -451,6 +581,8 @@ save_xml_keymap (gchar * filename, keymap * the_keymap)
   doc = xmlNewDoc ((xmlChar *) "1.0");
   doc->xmlRootNode = parent = xmlNewDocNode (doc, NULL, (xmlChar *) "Denemo",
 					     NULL);
+
+
   child = xmlNewChild (parent, NULL, (xmlChar *) "commands", NULL);
 
   xmlNewTextChild (child, NULL, (xmlChar *) "title", (xmlChar *) "A Denemo Keymap");
@@ -458,23 +590,36 @@ save_xml_keymap (gchar * filename, keymap * the_keymap)
 
   parent = xmlNewChild (child, NULL, (xmlChar *) "map", NULL);
 
+  child = xmlNewChild (parent, NULL, (xmlChar *) "cursors", NULL);
+
+  g_hash_table_foreach(Denemo.map->cursors, output_pointer_shortcut, child);
+
+
   for (i = 0; i < keymap_size(the_keymap); i++)
     {
+
      
       gpointer action = (gpointer)lookup_action_from_idx(the_keymap, i);
       gchar *scheme = action?g_object_get_data(action, "scheme"):NULL;
-      
+      gboolean deleted = (gboolean) (action?g_object_get_data(action, "deleted"):NULL);
+      gboolean hidden = (gboolean) (action?g_object_get_data(action, "hidden"):NULL);
+      if(deleted && scheme)
+	continue;
+
       child = xmlNewChild (parent, NULL, (xmlChar *) "row", NULL);
-      
+
+			
       gchar *name = (gchar*)lookup_name_from_idx(the_keymap, i);
 #ifdef DEBUG
       g_print ("%s \n", name);
 #endif	
       xmlNewTextChild (child, NULL, (xmlChar *) "action",
 		       (xmlChar *) name);  
+      if(hidden)
+	xmlNewTextChild (child, NULL, (xmlChar *) "hidden", "true");
       if(scheme) 	
 	xmlNewTextChild (child, NULL, (xmlChar *) "scheme",
-			 (xmlChar *) scheme);
+			 /* (xmlChar *) scheme*/ (xmlChar*)"");
       
       gchar *menupath = action?g_object_get_data(action, "menupath"):NULL;
       if(menupath)
@@ -492,7 +637,6 @@ save_xml_keymap (gchar * filename, keymap * the_keymap)
 	xmlNewTextChild (child, NULL, (xmlChar *) "tooltip",
 			 (xmlChar *) tooltip);
       
-      
       keymap_foreach_command_binding (the_keymap, i,
 				      (GFunc) write_xml_keybinding_info, child);
       
