@@ -3,6 +3,7 @@
  * (c) 1999-2005 Matthew Hiller
  */
 
+#include <string.h>
 
 #include "kbd-custom.h"
 #include "keyresponses.h"
@@ -13,13 +14,18 @@
 #include "view.h"
 
 
+
+
+
+
+
 static GdkEventKey ** divert_key_event;/* Non null if key events are being intercepted by a function
 					* (which is running a gtk_mail_loop() for this reason).
 					* return TRUE if a key press successfully captured
 					* in which case the kyval, state pair are returned
 					*/
 
-gboolean intercept_scorearea_keypress (gint *keyval, gint *state) {
+gboolean intercept_scorearea_keypress (GdkEventKey *pevent) {
   if(divert_key_event) {
     warningdialog("Recursive key capture not possible!");/* we could make a stack of them instead ... */
     return FALSE;
@@ -28,32 +34,89 @@ gboolean intercept_scorearea_keypress (gint *keyval, gint *state) {
   divert_key_event = &event;
   gtk_main();
   divert_key_event = NULL;
-  *keyval = event->keyval;
-  *state = dnm_sanitize_key_state(event);
-  //g_object_unref(event);
+  // *keyval = event->keyval;
+  // *state = dnm_sanitize_key_state(event);
+  memcpy(pevent, event, sizeof(GdkEventKey));
+  return TRUE;
 }
 
+
+
+static guint lock_mask(gint keyval) {
+
+  if((keyval==GDK_Shift_L)||
+     (keyval==GDK_Shift_R))
+    return GDK_SHIFT_MASK;
+
+  if(keyval==GDK_Caps_Lock)
+    return GDK_LOCK_MASK;
+    
+  if((keyval==GDK_Control_L)||
+     (keyval==GDK_Control_R))
+    return GDK_CONTROL_MASK;
+
+
+  if(keyval==GDK_Alt_L)
+    return GDK_MOD1_MASK;
+
+
+
+  if(keyval==GDK_Num_Lock)
+    return GDK_MOD2_MASK;
+
+     /*penguin/windows */
+  if((keyval==GDK_Super_L)||
+     (keyval==GDK_Super_R))
+    return GDK_MOD4_MASK;
+
+  if(keyval==GDK_Alt_R || GDK_ISO_Level3_Shift )
+    return GDK_MOD5_MASK;
+
+  return 0;
+
+}
+
+
+
+/**
+ * keyrelease event callback 
+ * sets cursor if a modifier
+ */
+
+gint
+scorearea_keyrelease_event (GtkWidget * widget, GdkEventKey * event)
+{
+  // set_cursor_for(keyrelease_modify(event->state), event->keyval);
+  gint state;
+  if((event->keyval==GDK_Caps_Lock) || (event->keyval==GDK_Num_Lock))
+    return TRUE;
+  state = (lock_mask(event->keyval)^event->state);
+  set_cursor_for(state);
+  return TRUE;
+}
 /**
  * keypress event callback 
  * looks up the key press and executes the correct function
  */
 
-int
+gint
 scorearea_keypress_event (GtkWidget * widget, GdkEventKey * event)
 {
   DenemoGUI *gui = Denemo.gui;
-  keymap *the_keymap = Denemo.commands;
+  keymap *the_keymap = Denemo.map;
   if(divert_key_event && !isModifier(event)) {
     *divert_key_event = event;
     //g_object_ref(event); FIXME do we need to keep it around?
     gtk_main_quit();
-    return 1;
+    return TRUE;
   }
-
+  gint state;
+  state = (lock_mask(event->keyval)^event->state);
+  if(state || ((event->keyval==GDK_Caps_Lock) || (event->keyval==GDK_Num_Lock)))
+    set_cursor_for(state); // MUST LOOK AHEAD to state after keypress HERE CATCH modifiers and set the cursor for them.....
   /* Look up the keystroke in the keymap and execute the appropriate
    * function */
-  gint command_idx = lookup_keybinding (the_keymap, event->keyval,
-			       dnm_sanitize_key_state(event));
+  gint command_idx = lookup_command_for_keyevent(event);
   if (command_idx != -1) {
     const gchar *command_name =
       lookup_name_from_idx (the_keymap, command_idx);
@@ -61,14 +124,15 @@ scorearea_keypress_event (GtkWidget * widget, GdkEventKey * event)
       Denemo.last_keyval = event->keyval;
       Denemo.last_keystate =  dnm_sanitize_key_state(event);
       execute_callback_from_name(the_keymap, command_name);
+      gui = Denemo.gui;
       displayhelper (gui);
       gtk_widget_draw (gui->scorearea, NULL);   
-      return 1;
+      return TRUE;
     } else {
       g_warning("No action %i has no name", command_idx);
     }
   }
-  return 0;
+  return TRUE;
 }
 
 /**
@@ -272,7 +336,7 @@ default_mode (DenemoGUI * gui)
 void
 rest_toggle_key (DenemoGUI * gui)
 {
-GtkAction *mode = gtk_ui_manager_get_action (Denemo.ui_manager, "/MainMenu/EntryMenu/Rest");
+GtkAction *mode = gtk_ui_manager_get_action (Denemo.ui_manager, "/MainMenu/ModeMenu/Rest");
   gtk_action_activate(mode);
 
 }
@@ -284,7 +348,7 @@ GtkAction *mode = gtk_ui_manager_get_action (Denemo.ui_manager, "/MainMenu/Entry
 void
 toggle_blank (DenemoGUI * gui)
 {
-GtkAction *mode = gtk_ui_manager_get_action (Denemo.ui_manager, "/MainMenu/EntryMenu/Blank");
+GtkAction *mode = gtk_ui_manager_get_action (Denemo.ui_manager, "/MainMenu/ModeMenu/Blank");
   gtk_action_activate(mode);
 }
 
@@ -534,20 +598,20 @@ deletepreviousobject(DenemoGUI * gui)
   if (gui->si->cursor_x)
     {
       /* Then move the cursor back */
-      cursorleft (gui);
+      cursorleft (NULL);
       /* And delete */
       deleteobject (gui);
     }
   else
     {/* go to the previous measure and start deleting there */
       if(gui->si->currentmeasure->prev) {
-	measureleft(gui);
+	measureleft(NULL);
 	while (gui->si->currentobject && (gui->si->currentobject->next))
 	  {
 	    gui->si->currentobject = gui->si->currentobject->next;
 	    gui->si->cursor_x++;
 	  }
-	cursorright(gui);
+	cursorright(NULL);
 	if(gui->si->currentobject)
 	  deletepreviousobject(gui);
       }
