@@ -13,11 +13,9 @@
 
 
 void
-lily_directive_postfix (GtkAction *action, gpointer param);
+lily_directive_attach_note (GtkAction *action, gpointer param);
 void
-lily_directive_insert (GtkAction *action, gpointer param);
-
-#ifdef DENEMO_DYNAMIC_MENU_ITEMS
-void myactivate (GtkAction *action, gpointer param);
-#endif
+lily_directive_attach_chord (GtkAction *action, gpointer param);
+void
+lily_directive_insert (GtkAction *action, DenemoScriptParam *param);
 #endif
