@@ -45,7 +45,7 @@ midi_seq *sq;
 #include "utils.h"
 #include "alsaplayback.h"
 
-int openfile (gchar * name);
+
 
 static const GtkStockItem denemo_stock_items[] = {
   {"denemo-staccato", N_("Staccato"), (GdkModifierType) 0, 0, NULL},
@@ -406,7 +406,8 @@ main (int argc, char *argv[])
   GError *error = NULL;
   /* gtk initialization */
   gtk_init (&argc, &argv);
-  //DenemoGUI *gui = (DenemoGUI *) g_malloc0 (sizeof (DenemoGUI));
+
+  
 
 //#ifdef G_OS_WIN32
 //  /* workaround necessary for compilation on Cygwin */
@@ -420,8 +421,13 @@ main (int argc, char *argv[])
 
   /* initialization of directory relocatability */
   initdir();
-
-
+#ifdef G_OS_WIN32
+  gchar *scmcode = g_build_filename(get_data_dir(), "share", "guile", NULL);
+  if(g_file_test(scmcode, G_FILE_TEST_EXISTS)) {
+  g_setenv("GUILE_LOAD_PATH", scmcode, TRUE);
+  g_print("Set environment variable GUILE_LOAD_PATH to %s\n", scmcode);
+  } else warningdialog("You may need to set GUILE_LOAD_PATH to the directory where you have ice9 installed\n");
+#endif
   /* locale initialization */
   //setlocale (LC_CTYPE, "");
   //setlocale (LC_MESSAGES, "");
@@ -432,20 +438,20 @@ main (int argc, char *argv[])
   textdomain (PACKAGE);
 
   register_stock_items ();
-  g_print("Calling scm boot guile with %d and %p\n", argc, argv);
+  //g_print("Calling scm boot guile with %d and %p\n", argc, argv);
  
-    scm_boot_guile (argc, argv, inner_main, NULL);
+  scm_boot_guile (argc, argv, inner_main, NULL);
   return 0;
 }
 
 
-int
-openfile (gchar * name)
+static int
+openfile (gchar * name, gboolean template)
 {
   GList *tmp = g_list_nth (Denemo.guis, 0);
 
   DenemoGUI *gui = (DenemoGUI *) tmp->data;
-  gint result = open_for_real (name, gui, FALSE, FALSE);
+  gint result = open_for_real (name, gui, template, REPLACE_SCORE);
   //gui->si->readonly = FALSE;
   //si->readonly = TRUE;
   return result;
@@ -525,6 +531,8 @@ COPYING for details.\n\n") ;
   //register_stock_items ();
   /* Following calls were made previously in newview. However I think they are
    * global, and should be done once and for all when the application opens
+   * they are now done after initializing guile, at the start of inner_main()
+   * which then calls this procedure.
    */
   /* Initialize preferences */
   //initprefs();
@@ -535,7 +543,20 @@ COPYING for details.\n\n") ;
   
   /* audio initialization */
   ext_init ();                  /* external players (midi...) */
-  midi_init ();                 /* internal (not working yet) */
+
+  /* Immediate Playback */
+  if(Denemo.prefs.immediateplayback) {
+    if( midi_init ()  )  {           /* Opens Denemo.prefs.sequencer, if this is set to an empty
+				 string then the open fails and direct audio out is used for 
+				immediate playback */
+      //g_print("Initializing audio out\n");
+      init_audio_out();
+
+    }
+  }
+
+
+
   //DenemoPrefs prefs;
   //  readxmlprefs("src/denemorc", &prefs);
 
@@ -591,7 +612,7 @@ COPYING for details.\n\n") ;
           switch (result)
             {
             case GTK_RESPONSE_ACCEPT:
-              openfile (name);
+              openfile (name, FALSE);
               g_remove (name);
               break;
             case GTK_RESPONSE_CANCEL:
@@ -617,7 +638,14 @@ COPYING for details.\n\n") ;
   if (dir)
     g_dir_close (dir);
 
-
+  gchar *init_file;
+  init_file = g_build_filename(get_data_dir (), "actions", "init.denemo", NULL);
+  if (openfile (init_file, TRUE) == -1)
+    g_warning("Denemo Scheme initialization file %s not found", init_file);
+  init_file = g_build_filename(locatedotdenemo (), "actions", "init.denemo", NULL);
+  (void)openfile (init_file, TRUE);
+  deleteSchemeText();
+  g_free(init_file);
   /* Open a file, if it was specified on the command line. Note
    * that this had to be done after the window was created, otherwise
    * there wouldn't have been a titlebar to set. Also note that
@@ -626,10 +654,10 @@ COPYING for details.\n\n") ;
 
   if (optind < argc)
     {
-      if (openfile (argv[optind]) == -1)
+      if (openfile (argv[optind], FALSE) == -1)
         {
           g_print ("Attempt to read in file %s failed\n", argv[optind]);
           return 1;
-        }
+        } 
     }
 }
