@@ -248,7 +248,7 @@ calcticksperbeat (gint time1, gint time2)
  * calculatebeamsandstemdirs is invoked, and would share code besides, so
  * that's precisely where it is invoked 
  */
-void
+static void
 settickvalsinmeasure (objnode * theobjs, gint ticksperbeat)
 {
   gint numerator = 1, denominator = 1;
@@ -269,22 +269,24 @@ settickvalsinmeasure (objnode * theobjs, gint ticksperbeat)
 		
 		if (theobj->type == CHORD)
 		{
-			if (in_tuplet)
-			{
-				set_tuplefied_numticks (theobj, numerator, denominator);
-				basic_ticks_in_tuplet_group += theobj->basic_durinticks;
-			}
-			else if (in_grace)
-			{
-				set_grace_numticks (theobj, 8);
-				basic_ticks_in_grace_group += theobj->basic_durinticks;
-			}
-			else
-			{
-				set_tuplefied_numticks (theobj, 1, 1);
-				set_grace_numticks (theobj, 1);
-				ticks_so_far += theobj->durinticks;
-			}
+		  if (in_tuplet)
+		    {
+		      set_tuplefied_numticks (theobj, numerator, denominator);
+		      basic_ticks_in_tuplet_group += theobj->basic_durinticks;
+		    }
+		  else if (in_grace)
+		    {
+		      ((chord *)theobj->object)->is_grace = TRUE;
+		      set_grace_numticks (theobj, 1/*8*/);/*8 =  sets durinticks too small */
+		      basic_ticks_in_grace_group += theobj->basic_durinticks;
+		    }
+		  else
+		    {
+		      ((chord *)theobj->object)->is_grace = FALSE;
+		      set_tuplefied_numticks (theobj, 1, 1);//These two overwrite each other...
+		      set_grace_numticks (theobj, 1);
+		      ticks_so_far += theobj->durinticks;
+		    }
 		}
 		else if (theobj->type == TUPOPEN)
 		{
@@ -310,13 +312,20 @@ settickvalsinmeasure (objnode * theobjs, gint ticksperbeat)
 		else if (theobj->type == GRACE_END)
 		{
 			in_grace = FALSE;
-			/*      ticks_so_far += basic_ticks_in_grace_group;  RTS - this is just a mistake I think */
+			// anything here makes next note back up over grace end ... ticks_so_far -= 8;
+			//basic_ticks_in_grace_group accumulates the basic_ticks of the notes
+			// after grace start
+			//ticks_so_far += basic_ticks_in_grace_group;//???? this affects furthest
 			basic_ticks_in_grace_group = 0;
 		}
 		
 		theobj->starttickofnextnote =
 		ticks_so_far + (basic_ticks_in_tuplet_group * numerator
 						/ denominator) + basic_ticks_in_grace_group;
+
+		//this goes up too fast for grace notes...
+
+		//g_print("start tick next %d\n", 	theobj->starttickofnextnote);
     }
 }
 
@@ -425,9 +434,11 @@ calculatebeamsandstemdirs (objnode * theobjs, gint * pclef, gint * time1,
       if (theobj->type != CHORD || isrest)
 	{
 	  /* A non-chord or rest always breaks up a beam group */
-	  /* I may want to adjust rest behavior to something else at some point */
-	  theobj->isstart_beamgroup = TRUE;
-	  theobj->isend_beamgroup = TRUE;
+	  /* LilyPond directives can have their own behaviour,
+	   starting with *not* forcing beam breaks */
+	  /* if(theobj->type != LILYDIRECTIVE) */{
+	    theobj->isstart_beamgroup = TRUE;
+	    theobj->isend_beamgroup = TRUE;}
 
 	  switch (theobj->type)
 	    {
@@ -658,7 +669,7 @@ showwhichaccidentals (objnode * theobjs, gint initialnum, gint * initialaccs)
 		/* A new accidental not present in the original chord */
 		thetone->showaccidental =
 		  ((chord *) theobj->object)->hasanacc = TRUE;
-	      else if (thetone->directive && (*thetone->directive->str=='!'|| *thetone->directive->str=='?'))
+	      else if (thetone->postfix && (*thetone->postfix->str=='!'|| *thetone->postfix->str=='?'))
 		thetone->showaccidental =
 		  ((chord *) theobj->object)->hasanacc = TRUE;
 	      else
