@@ -18,14 +18,15 @@
 static GtkWidget *PR_window = NULL;/* a top level window for controlling pitch-recognition entry.
 			      We do not create one of these for each view (ie each DenemoGUI object, ie each score) because there is only one audio input source being used, so we would have to cope with resource contention issues, there is just no point. */
 static DenemoGUI *PR_gui; /* the gui for which the pitch recognition has been set up */
-static gboolean PR_insert; /* whether the notes should be inserted directly in the score or put in a list for controlled insertion */
-static gboolean PR_continuous; /* whether to go on to the next measure once overlay for current measure is full */
+
+
 gint PR_click;/*volume of a audible warning of next measure or extra tones in measure */
 static gboolean PR_tuning; /* whether the notes should be analyzed to refine the frequency determination
 			    for tuning instruments etc */
 static GtkWidget *PR_notelabel = NULL;
 static GtkWidget *PR_deviation = NULL;
 static GtkWidget *PR_indicator = NULL;
+static GtkWidget *PR_label=NULL;
 static gdouble PR_target_pitch = 440.0;
 static gdouble PR_accurate_pitch = 0.0;
 
@@ -178,7 +179,7 @@ static temperament WerckmeisterIV = {
   }
 };
 
-static temperament *PR_temperament; /* the currently used temperament */
+static temperament *PR_temperament=&Equal; /* the currently used temperament */
 
 static temperament *temperaments[] = {&Equal, &Meantone, &WerckmeisterIV, &Lehman};
 
@@ -257,6 +258,11 @@ static gchar * notenames(gpointer p) {
   return str;
 }
 
+static gchar *nameof(gint notenumber) {
+ return g_strdup_printf("%c%s", step_name(PR_temperament->notepitches[notenumber].spec.step), alteration_name(PR_temperament->notepitches[notenumber].spec.alteration));
+}
+
+
 
 /* returns an opaque id for the user's default temperament
  FIXME user prefs */
@@ -291,16 +297,19 @@ static void sharpen(GtkButton *button, GtkWidget *label) {
 #undef g
   PR_temperament->sharp = PR_temperament->flat;
   PR_temperament->flat = (PR_temperament->flat+7)%12;
-  gchar *names = notenames(PR_temperament);
-  gtk_label_set_markup((GtkLabel*)label, names);
-  g_free(names);
-  switch_back_to_main_window();
+  if(PR_window) {
+    gchar *names = notenames(PR_temperament);
+    gtk_label_set_markup((GtkLabel*)label, names);
+    g_free(names);
+    switch_back_to_main_window();
+  }
   return;
 }
 
 
 
 
+
 static void flatten(GtkButton *button, GtkWidget *label) {
 
 #define s (PR_temperament->notepitches[PR_temperament->sharp].spec)
@@ -316,14 +325,46 @@ if(t.alteration-1<-2)
 #undef t
   PR_temperament->flat = PR_temperament->sharp;
   PR_temperament->sharp = (PR_temperament->sharp+5)%12;
+  if(PR_window) {
   gchar *names = notenames(PR_temperament);
   gtk_label_set_markup((GtkLabel*)label, names);
   g_free(names);
   switch_back_to_main_window();
+  }
   return;
 }
 
+static void enharmonic_step (gboolean sharp) {
+  gchar *sharpestname, *flattestname;
+  if(sharp)
+    sharpen(NULL, PR_label);
+  else
+    flatten(NULL, PR_label);
+  GtkAction *sharpaction = gtk_ui_manager_get_action (Denemo.ui_manager, "/MainMenu/InputMenu/SharpenEnharmonicSet");
+  GtkAction *flataction = gtk_ui_manager_get_action (Denemo.ui_manager, "/MainMenu/InputMenu/FlattenEnharmonicSet");
 
+  sharpestname = nameof(PR_temperament->sharp);
+  flattestname = nameof(PR_temperament->flat);
+  gchar *label = g_strdup_printf("(%s) Step Sharper", sharpestname);
+  GValue a = {0};
+  g_value_init (&a, G_TYPE_STRING);
+  g_value_set_string (&a, label);
+  g_object_set_property(G_OBJECT(sharpaction), "label", &a);
+  g_free(label);
+  label = g_strdup_printf("(%s) Step Flatter", flattestname);
+  g_value_set_string (&a, label);
+  g_object_set_property(G_OBJECT(flataction), "label", &a);
+  g_free(label);
+  g_free(sharpestname);
+  g_free(flattestname);
+}
+
+void set_sharper(GtkAction *action, gpointer param) {
+  enharmonic_step (TRUE);
+}
+void set_flatter(GtkAction *action, gpointer param) {
+  enharmonic_step (FALSE);
+}
 static void sound_click(void) {
   if(PR_click)
     gdk_beep();
@@ -377,7 +418,7 @@ static gboolean  apply_tones(DenemoScore *si) {
 	    modify_note((chord*)theobj->object, mid_c_offset, thetone->enshift, dclef);
 	    tone_stored=TRUE;
 	    if(!si->cursor_appending)
-	      cursorright(PR_gui);
+	      cursorright(Denemo.gui);
 	    store_el = store_el->next;
 
 	  }// tone available
@@ -392,7 +433,7 @@ static gboolean  apply_tones(DenemoScore *si) {
 	if(tone_stored && curobj==NULL && curmeasure->next)
 	 ret = TRUE;
       }// while objects in measure
-    if(store_el && !PR_continuous) 
+    if(store_el && !Denemo.prefs.continuous) 
       sound_click();//extra tones in measure
     showwhichaccidentals ((objnode *) si->currentmeasure->data,
 			  si->curmeasurekey, si->curmeasureaccs);
@@ -447,9 +488,9 @@ static void enter_tone_in_store (DenemoGUI *gui, notepitch * found, gint octave)
   store = put_tone(store, gui->si->currentmeasurenum - 1, thetone);
   nextmeasure = apply_tones(gui->si);
   displayhelper (gui);
-  if(PR_continuous && nextmeasure) {
+  if(Denemo.prefs.continuous && nextmeasure) {
     sound_click();
-    measureright(gui);
+    measureright(NULL);
   }  
 #undef store
 }
@@ -528,7 +569,7 @@ gboolean delete_tone(DenemoScore *si, chord *thechord) {
   si->cursor_appending = keepa;
   calcmarkboundaries (si);
 
-  displayhelper (PR_gui);
+  displayhelper (Denemo.gui);
   return TRUE;
   } else {
     return FALSE;
@@ -573,6 +614,8 @@ static void display_pitch(double note, DenemoGUI *gui) {
   gint octave;
   double deviation;
   temperament *t = (temperament*)PR_temperament;
+  if(!GTK_IS_WINDOW(PR_window))
+    return;
     notepitch *found = determine_note(note, t , &octave, &deviation);
     if(found) {
       int i;
@@ -651,7 +694,9 @@ gint pitchentry(DenemoGUI *gui) {
       // Enter the note in the score
       if(!PR_tuning){
 	display_pitch(note, gui);
-	if(PR_insert)
+	if(gui->input_source==INPUTMIDI)
+	  play_pitch(found->pitch * (pow(2,(octave))), 0.3, 0.5, 0);
+	if(!Denemo.prefs.overlays)
 	  enter_note_in_score(gui, found, octave);
 	else
 	  enter_tone_in_store(gui, found, octave);
@@ -665,9 +710,9 @@ gint pitchentry(DenemoGUI *gui) {
 
 
 
-  // toggle PR_continuous advance to next measure or not
+  // toggle continuous advance to next measure or not
 static void toggle_continuous(GtkButton *button, DenemoGUI *gui ) {
-  PR_continuous = !PR_continuous;
+  Denemo.prefs.continuous = !Denemo.prefs.continuous;
   switch_back_to_main_window();
 }
 
@@ -721,32 +766,28 @@ static void frequency_smoothing(GtkSpinButton *widget, gpointer data){
   set_frequency_smoothing(m);
 }
 
-/* destroy PR_window, and hence terminate the pitch recog subsystem */
-static void stop_PR_controls(void) {
-  if(PR_window)
-    gtk_widget_destroy(PR_window);
-  PR_window = NULL;
-}
 
 
-/* stop_pitch_recognition
-   stop audio and aubio
+/* stop_pitch_input
+   if not midi stop audio and aubio
  */
-int stop_pitch_recognition(void) {
-  if(PR_gui==NULL)// when closing a second gui after the first has dismissed PR
-    return;
+int stop_pitch_input(void) {
+  DenemoGUI *gui = Denemo.gui;
   if(PR_timer)
     g_source_remove(PR_timer);
-  //  if(PR_oldkeymap) // should always be true!
-  //    PR_gui->prefs->the_keymap = PR_oldkeymap;
+
   if(PR_enter) 
-    g_signal_handler_disconnect (PR_gui->scorearea, PR_enter);
+    g_signal_handler_disconnect (Denemo.gui->scorearea, PR_enter);
   if(PR_leave) 
-    g_signal_handler_disconnect (PR_gui->scorearea, PR_leave);
+    g_signal_handler_disconnect (Denemo.gui->scorearea, PR_leave);
    PR_timer = PR_enter = PR_leave = 0;
 
-   terminate_pitch_recognition();
-   if(PR_window) { 
+   if(gui->input_source==INPUTAUDIO)
+      terminate_pitch_recognition();
+   else
+     stop_midi_input();
+   clear_tone_store(NULL, gui);
+   if(GTK_IS_WINDOW(PR_window)) { 
      GtkWidget *temp = PR_window; PR_window = NULL, gtk_widget_destroy(temp);
    }
   PR_gui = NULL;
@@ -754,13 +795,22 @@ int stop_pitch_recognition(void) {
   return 0;
 }
 
-static gboolean stop_pitch_recognition_callback(GtkWidget *unused, DenemoGUI *gui){
+static void   activate_action(gchar *path) {
+   GtkAction *a;
+   a = gtk_ui_manager_get_action (Denemo.ui_manager, path);
+   if(a)
+   gtk_action_activate(a);
+   else 
+     g_warning("Internal error, denemogui.xml out of step with literal %s in %s\n", path, __FILE__);
+ }
+
+
+static gboolean window_destroy_callback(void){
 if(PR_gui==NULL)
     return FALSE;
-GtkWidget * widget = gtk_ui_manager_get_widget (Denemo.ui_manager, "/RhythmToolBar/TogglePitchRecognition");
+ activate_action( "/MainMenu/InputMenu/KeyboardOnly");
  PR_window=NULL;
- gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON(widget),  FALSE);
- clear_tone_store(NULL, gui);
+ clear_tone_store(NULL, Denemo.gui);
  return FALSE;
 }
 
@@ -789,16 +839,14 @@ static void change_click_volume(GtkSpinButton *widget){
 
 static void change_timer_rate(GtkSpinButton *widget, DenemoGUI *gui){
   PR_time = (guint)gtk_spin_button_get_value(widget);
-  start_pitch_recognition(gui);//FIXME do not call the whole of start_pitch_recognition, just the timer setting bit???
+  start_pitch_input();//FIXME do not call the whole of start_pitch_recognition, just the timer setting bit???
   switch_back_to_main_window();
 }
 
-  // toggle PR_insert to show where the notes detected should go, and start correct entry mode
+  // toggle Denemo.prefs.overlays to show where the notes detected should go
 static void toggle_insert(GtkButton *button, DenemoGUI *gui) {
-  GtkAction *action;
-  action = gtk_ui_manager_get_action (Denemo.ui_manager, PR_insert?"/MainMenu/EntryMenu/EditMode": "/MainMenu/EntryMenu/InsertMode");
-  gtk_action_activate(action);
-  PR_insert = !PR_insert;
+  Denemo.prefs.overlays = !Denemo.prefs.overlays;
+  clear_tone_store(NULL, Denemo.gui);
   switch_back_to_main_window();
 }
 
@@ -873,90 +921,103 @@ static void create_pitch_recognition_window(DenemoGUI *gui) {
   GtkWidget *button;
   GtkWidget *frame;
   GtkWidget *label;
-  if(PR_window) {
+  GtkAdjustment *spinner_adj;
+  GtkWidget *spinner;
+  if(GTK_IS_WINDOW(PR_window)) {
     g_warning("unexpected call");
     return;
   }
-  PR_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);/* FIXME catch a destroy on this window
-						   and turn off pitch detection, returning to unitialized state */
+  PR_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
-  gtk_window_set_title (GTK_WINDOW (PR_window), "Pitch Recognition Control");
+  gtk_window_set_title (GTK_WINDOW (PR_window), "Pitch Input Control");
   g_signal_connect (G_OBJECT (PR_window), "destroy",
-		    G_CALLBACK (stop_pitch_recognition_callback), gui); 
+		    G_CALLBACK (window_destroy_callback), NULL); 
   GtkWidget *main_vbox = gtk_vbox_new (FALSE, 1);
   gtk_container_border_width (GTK_CONTAINER (main_vbox), 1);
   gtk_container_add (GTK_CONTAINER (PR_window), main_vbox);
+  if(gui->input_source==INPUTAUDIO) {
+    frame = gtk_frame_new( "Mode");
+    gtk_container_add (GTK_CONTAINER (main_vbox), frame);
+    hbox = gtk_hbox_new (FALSE, 1);
+    gtk_container_add (GTK_CONTAINER (frame), hbox);
 
-  frame = gtk_frame_new( "Mode");
-  gtk_container_add (GTK_CONTAINER (main_vbox), frame);
-  hbox = gtk_hbox_new (FALSE, 1);
-  gtk_container_add (GTK_CONTAINER (frame), hbox);
+    GtkWidget *vbox = gtk_vbox_new (FALSE, 1);
+    gtk_box_pack_start (GTK_BOX (hbox), vbox,
+			TRUE, TRUE, 0);
+    GtkWidget *radio_button = gtk_radio_button_new_with_label(NULL, "Overlay Pitches");
 
-  GtkWidget *vbox = gtk_vbox_new (FALSE, 1);
-  gtk_box_pack_start (GTK_BOX (hbox), vbox,
-		      TRUE, TRUE, 0);
-  GtkWidget *radio_button = gtk_radio_button_new_with_label(NULL, "Overlay Pitches");
+    g_signal_connect (G_OBJECT (radio_button), "toggled",
+		      G_CALLBACK (toggle_insert), gui);
+    button = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio_button), "Insert Notes");
 
-  g_signal_connect (G_OBJECT (radio_button), "toggled",
-		    G_CALLBACK (toggle_insert), gui);
-  button = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio_button), "Insert Notes");
+    //g_print("Overlays %d\n", Denemo.prefs.overlays);
+    if(Denemo.prefs.overlays) {
+      //gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(button), FALSE);
+      gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radio_button), TRUE);
+    }
+    else {
+      Denemo.prefs.overlays = !Denemo.prefs.overlays;
+      gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(button), TRUE);
+     
+      //gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radio_button), FALSE);
+    }
+    //g_print("Overlays after button setting %d\n", Denemo.prefs.overlays);
+    gtk_box_pack_start (GTK_BOX (vbox), button,
+			TRUE, TRUE, 0);/* no need for callback */
 
-  PR_insert = TRUE;
-  toggle_insert(NULL, gui);
-  gtk_box_pack_start (GTK_BOX (vbox), button,
-		      TRUE, TRUE, 0);/* no need for callback */
+    gtk_box_pack_start (GTK_BOX (vbox), radio_button,
+			TRUE, TRUE, 0);
 
-  gtk_box_pack_start (GTK_BOX (vbox), radio_button,
-		      TRUE, TRUE, 0);
 
-  button = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio_button), "Tuning");
-  g_signal_connect (G_OBJECT (button), "toggled",
-		    G_CALLBACK (toggle_tuning), gui);
-  gtk_box_pack_start (GTK_BOX (vbox), button,
-		      TRUE, TRUE, 0);
+    button = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio_button), "Tuning");
+    g_signal_connect (G_OBJECT (button), "toggled",
+		      G_CALLBACK (toggle_tuning), gui);
+    gtk_box_pack_start (GTK_BOX (vbox), button,
+			TRUE, TRUE, 0);
 
-  frame = gtk_frame_new( "Overlay Pitches");
-  gtk_container_add (GTK_CONTAINER (hbox), frame);
 
-  GtkWidget *vbox2 = gtk_vbox_new (FALSE, 1);
-  gtk_container_add (GTK_CONTAINER (frame), vbox2);
+    frame = gtk_frame_new( "Overlay Pitches");
+    gtk_container_add (GTK_CONTAINER (hbox), frame);
 
-  hbox2 = gtk_hbox_new (FALSE, 1);
+    GtkWidget *vbox2 = gtk_vbox_new (FALSE, 1);
+    gtk_container_add (GTK_CONTAINER (frame), vbox2);
 
-  gtk_box_pack_start (GTK_BOX (vbox2), hbox2,
-		      TRUE, TRUE, 0);
-  
+    hbox2 = gtk_hbox_new (FALSE, 1);
+
+    gtk_box_pack_start (GTK_BOX (vbox2), hbox2,
+			TRUE, TRUE, 0);
   
+    button = gtk_button_new_with_label("Clear Overlay"); //FIXME make this a proxy for the ClearOverlay action ??
+    gtk_box_pack_start (GTK_BOX (hbox2), button,
+			TRUE, TRUE, 0);
+    g_signal_connect (G_OBJECT (button), "clicked",
+		      G_CALLBACK (clear_tone_store), gui);
+    hbox2 = gtk_hbox_new (FALSE, 1);
 
-  button = gtk_button_new_with_label("Clear Overlay"); //FIXME make this a proxy for the ClearOverlay action ??
-  gtk_box_pack_start (GTK_BOX (hbox2), button,
-		      TRUE, TRUE, 0);
-  g_signal_connect (G_OBJECT (button), "clicked",
-		    G_CALLBACK (clear_tone_store), gui);
-  hbox2 = gtk_hbox_new (FALSE, 1);
-
-  gtk_box_pack_start (GTK_BOX (vbox2), hbox2,
-		      TRUE, TRUE, 0);
-  button = gtk_check_button_new_with_label("Continuous");
-  gtk_box_pack_start (GTK_BOX (hbox2), button,
-		      TRUE, TRUE, 0);
-  PR_continuous = FALSE;
-  g_signal_connect (G_OBJECT (button), "clicked",
-		    G_CALLBACK (toggle_continuous), gui);
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
-
-  label = gtk_label_new("Click Volume");
-  gtk_box_pack_start (GTK_BOX (hbox2), label, TRUE, TRUE, 0);
-  PR_click = 1;
-  GtkAdjustment *spinner_adj =
-    (GtkAdjustment *) gtk_adjustment_new ((double)PR_click, 0.0, 1.0,
-					   1.0, 1.0, 1.0);
-  GtkWidget *spinner = gtk_spin_button_new (spinner_adj, 1.0, 1);
-  gtk_box_pack_start (GTK_BOX (hbox2), spinner, TRUE, TRUE, 0);
-  g_signal_connect (G_OBJECT (spinner), "value-changed",
-		    G_CALLBACK (change_click_volume), NULL);
+    gtk_box_pack_start (GTK_BOX (vbox2), hbox2,
+			TRUE, TRUE, 0);
+    button = gtk_check_button_new_with_label("Continuous");
+    gtk_box_pack_start (GTK_BOX (hbox2), button,
+			TRUE, TRUE, 0);
 
+    g_signal_connect (G_OBJECT (button), "clicked",
+		      G_CALLBACK (toggle_continuous), gui);
+    if(Denemo.prefs.continuous){
+      Denemo.prefs.continuous = !Denemo.prefs.continuous;
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), !Denemo.prefs.continuous);
+    }
+    label = gtk_label_new("Click Volume");
+    gtk_box_pack_start (GTK_BOX (hbox2), label, TRUE, TRUE, 0);
+    PR_click = 1;
+    spinner_adj =
+      (GtkAdjustment *) gtk_adjustment_new ((double)PR_click, 0.0, 1.0,
+					    1.0, 1.0, 1.0);
+    spinner = gtk_spin_button_new (spinner_adj, 1.0, 1);
+    gtk_box_pack_start (GTK_BOX (hbox2), spinner, TRUE, TRUE, 0);
+    g_signal_connect (G_OBJECT (spinner), "value-changed",
+		      G_CALLBACK (change_click_volume), NULL);
 
+  }// if audio input allow use of overlay mechanism, MIDI can use MIDIAdvanceOnEdit filter.
 
 
   frame = gtk_frame_new( "Enharmonic selection");
@@ -964,11 +1025,11 @@ static void create_pitch_recognition_window(DenemoGUI *gui) {
   hbox = gtk_hbox_new (FALSE, 1);
   gtk_container_add (GTK_CONTAINER (frame), hbox);
   label = gtk_label_new("");
+  PR_label = label;
   button = gtk_button_new_with_label("flatten");
   gtk_box_pack_start (GTK_BOX (hbox), button,
 		      TRUE, TRUE, 0);
-  g_signal_connect (G_OBJECT (button), "clicked",
-		    G_CALLBACK (flatten), label);
+  gtk_action_connect_proxy(gtk_ui_manager_get_action (Denemo.ui_manager, "/MainMenu/InputMenu/FlattenEnharmonicSet"), button);
   gchar *names = notenames(PR_temperament);
   
   gtk_label_set_markup(GTK_LABEL(label),names);
@@ -979,8 +1040,7 @@ static void create_pitch_recognition_window(DenemoGUI *gui) {
   button = gtk_button_new_with_label("sharpen");
   gtk_box_pack_start (GTK_BOX (hbox), button,
 		      TRUE, TRUE, 0);
-  g_signal_connect (G_OBJECT (button), "clicked",
-		    G_CALLBACK (sharpen), label);
+  gtk_action_connect_proxy(gtk_ui_manager_get_action (Denemo.ui_manager, "/MainMenu/InputMenu/SharpenEnharmonicSet"), button);
 
   frame = gtk_frame_new( "Detected note");
   gtk_container_add (GTK_CONTAINER (main_vbox), frame);
@@ -993,6 +1053,10 @@ static void create_pitch_recognition_window(DenemoGUI *gui) {
   PR_deviation = gtk_label_new("0.0");
   gtk_box_pack_start (GTK_BOX (hbox), PR_deviation,
 		      TRUE, TRUE, 0);
+
+
+
+  if(gui->input_source==INPUTAUDIO) {
   /* spinners to select silence, threshold, smoothing */ 
   
   frame = gtk_frame_new( "Pitch Recognition Parameters");
@@ -1050,7 +1114,7 @@ static void create_pitch_recognition_window(DenemoGUI *gui) {
 		    G_CALLBACK (change_onset_detection), NULL);
 
 
-
+  }
   /* spinners to constrain the note values */
 
 
@@ -1098,8 +1162,10 @@ static void create_pitch_recognition_window(DenemoGUI *gui) {
   g_signal_connect (G_OBJECT (spinner), "value-changed",
 		    G_CALLBACK (change_greatest_interval), NULL);
 
-  /* options */
 
+
+  /* options */
+  if(gui->input_source==INPUTAUDIO) {
  frame = gtk_frame_new( "Input handling");
   gtk_container_add (GTK_CONTAINER (main_vbox), frame);
   hbox = gtk_hbox_new (FALSE, 1);
@@ -1170,7 +1236,7 @@ static void create_pitch_recognition_window(DenemoGUI *gui) {
   gtk_box_pack_start (GTK_BOX (hbox ),
 		      combobox, TRUE, TRUE, 0);
 
-
+  }
 
 /* now show the window, but leave the main window with the focus */
   gtk_window_set_focus_on_map((GtkWindow *)PR_window, FALSE);
@@ -1185,30 +1251,32 @@ static void create_pitch_recognition_window(DenemoGUI *gui) {
 }
 
 
-gint setup_pitch_recognition(DenemoGUI *gui){
-  if(PR_window) {
+gint setup_pitch_input(void){
+  DenemoGUI *gui = Denemo.gui;
+  if(GTK_IS_WINDOW(PR_window)) {
     gtk_window_present(GTK_WINDOW(PR_window));
     return 0;
   }
   if(PR_temperament==NULL)
     PR_temperament = default_temperament();
-  if(initialize_pitch_recognition()==0) {
-    set_silence(-90.0);
-    set_threshold(0.3);
-    set_smoothing(6.0);
-    transposition_required = 1.0;
-    lowest_pitch = DEFAULT_LOW;
-    highest_pitch = DEFAULT_HIGH;
-    repeated_notes_allowed = TRUE;
-    create_pitch_recognition_window(gui);
-    //read_PRkeymap(gui);
-    return 0;
+  if(gui->input_source==INPUTAUDIO?(initialize_pitch_recognition()==0):(init_midi_input()==0)) {
+    if(gui->input_source==INPUTAUDIO) {//FIXME these should be done at initialize_pitch_recognition time
+      set_silence(-90.0);
+      set_threshold(0.3);
+      set_smoothing(6.0);
+    }
   } else
     return -1;
+  transposition_required = 1.0;
+  lowest_pitch = DEFAULT_LOW;
+  highest_pitch = DEFAULT_HIGH;
+  repeated_notes_allowed = TRUE;
+  create_pitch_recognition_window(gui);
+  //read_PRkeymap(gui);
+  return 0;
 }
 
 
-
 static void 
 scorearea_set_active(GtkWidget *widget, GdkEventCrossing *event, DenemoGUI *gui) {
   PR_enable = TRUE; 
@@ -1219,22 +1287,23 @@ scorearea_set_inactive(GtkWidget *widget, GdkEventCrossing *event, DenemoGUI *gu
   PR_enable = FALSE; 
   gtk_widget_draw(gui->scorearea, NULL);
 }
-void start_pitch_recognition(DenemoGUI *gui) { 
+void start_pitch_input(void) { 
+  DenemoGUI *gui = Denemo.gui;
   if(PR_timer)
     g_source_remove(PR_timer);
 
-  PR_timer = g_timeout_add (PR_time, (GSourceFunc)pitchentry, gui);
-
-
-
+  PR_timer = g_timeout_add (PR_time, (GSourceFunc)pitchentry, Denemo.gui);
 
   if(PR_timer==0)
     g_error("Timer id 0 - if valid the code needs re-writing (documentation not clear)");
-  gtk_widget_add_events (gui->scorearea, GDK_LEAVE_NOTIFY_MASK | GDK_ENTER_NOTIFY_MASK);
-  PR_enter = g_signal_connect (G_OBJECT (gui->scorearea), "enter-notify-event",
+  if(gui->input_source==INPUTAUDIO) {/* for input from microphone avoid accidental activation by insisting on pointer being in the score drawing area */
+    gtk_widget_add_events (gui->scorearea, GDK_LEAVE_NOTIFY_MASK | GDK_ENTER_NOTIFY_MASK);
+    PR_enter = g_signal_connect (G_OBJECT (gui->scorearea), "enter-notify-event",
 			       G_CALLBACK (scorearea_set_active), (gpointer)gui);
-  PR_leave = g_signal_connect (G_OBJECT (gui->scorearea), "leave-notify-event",
+    PR_leave = g_signal_connect (G_OBJECT (gui->scorearea), "leave-notify-event",
 			       G_CALLBACK (scorearea_set_inactive), (gpointer)gui);
+  } else
+    PR_enable = TRUE;/* for midi input you are unlikely to enter notes by accident */
   PR_gui = gui;
 }
 
@@ -1246,3 +1315,29 @@ gboolean pitch_entry_active(DenemoGUI *gui) {
   return (PR_gui == gui) && PR_enable;
 }
 
+gchar *determine_interval(gint bass, gint harmony){
+  gint bass_octave, harmony_octave;
+  gdouble deviation;
+  gint semitones = harmony - bass;
+  gint *accs = ((DenemoStaff*)Denemo.gui->si->currentstaff->data)->skeyaccs;
+ notepitch bassnote = PR_temperament->notepitches[bass%12];
+ notepitch harmonynote = PR_temperament->notepitches[harmony%12];
+ gint interval =  harmonynote.spec.step - bassnote.spec.step + 1;
+ if(interval<2)interval += 7;
+ if(interval==2 && semitones>12) interval=9;
+ gint inflection = harmonynote.spec.alteration - accs[harmonynote.spec.step];
+
+
+ g_print("Bass %d harmony %d\nInterval is %d, semitones is %d cf (%d, %d)  \n keyaccs of bass note %d of harmony %d\ninflection %d\n", bass, harmony, interval, semitones, bassnote.spec.alteration, harmonynote.spec.alteration, accs[bassnote.spec.step], accs[harmonynote.spec.step], inflection);
+ gchar *modifier="";
+ if(interval==5 && semitones==6)
+   modifier = "/";
+     else 
+       if(inflection<0) modifier = "-";
+       else
+	 if(inflection>0) modifier = "+";
+ if(interval==3 && inflection)
+   return g_strdup_printf("%c%s", '_', modifier);
+ else
+   return g_strdup_printf("%d%s", interval, modifier);
+}
