@@ -50,13 +50,17 @@ static gint kill_timer(void){
   kill_id = 0;
 }
 
+
+
+
 /* start or restart an external midi player
  * trying avoid multiple instances of it
- * if action==NULL stop the playback
+ * if start==FALSE stop the playback rather than start it
  */
-void
-ext_midi_playback (GtkAction * action, DenemoGUI * gui)
+static void
+ext_midi_playback_control (gboolean start)
 {
+  DenemoGUI *gui = Denemo.gui;
   FILE *fp;
   int got, ok;
   GError *err = NULL;
@@ -77,7 +81,7 @@ ext_midi_playback (GtkAction * action, DenemoGUI * gui)
     {
       /* show a warning dialog */
       GtkWidget *dialog =
-        gtk_message_dialog_new (GTK_WINDOW (gui->window),
+        gtk_message_dialog_new (GTK_WINDOW (Denemo.window),
                                 GTK_DIALOG_DESTROY_WITH_PARENT,
                                 GTK_MESSAGE_WARNING,
                                 GTK_BUTTONS_OK,
@@ -115,7 +119,7 @@ ext_midi_playback (GtkAction * action, DenemoGUI * gui)
       g_debug ("ext_midi: killing player %x\n", playerpid);
       kill_process (playerpid);
     }
-  if(action==NULL) {
+  if(!start) {
     fclose (fp);
     g_free (pidpath);
     return;
@@ -174,9 +178,19 @@ ext_midi_playback (GtkAction * action, DenemoGUI * gui)
 }
 
 
-void stop_midi_playback (GtkAction * action, DenemoGUI * gui) {
-  ext_midi_playback (NULL, gui);
-  kill_timer();
+/* start or restart an external midi player
+ * trying avoid multiple instances of it
+ */
+void
+ext_midi_playback (GtkAction * action, gpointer param)
+{
+  ext_midi_playback_control (TRUE);
+
+}
+
+void stop_midi_playback (GtkAction * action, gpointer param) {
+ ext_midi_playback_control (FALSE);
+ kill_timer();
 }
 
 
