@@ -7,10 +7,8 @@ void
 playback_local (gpointer callback_data, guint callback_action,
 		GtkWidget * widget);
 void
-ext_midi_playback (GtkAction *action, DenemoGUI *gui);
+ext_midi_playback (GtkAction *action, gpointer param);
 void
-stop_midi_playback (GtkAction *action, DenemoGUI *gui);
+stop_midi_playback (GtkAction *action, gpointer param);
 
-void 
-stop_midi_playback (GtkAction * action, DenemoGUI * gui);
 #endif //PLAYBACKH
