@@ -31,11 +31,18 @@ struct callbackdata
   GtkWidget *checkrhythmpalette;
   GtkWidget *checkarticulationpalette;
   GtkWidget *autosaveentry;
+  GtkWidget *maxhistory;
   GtkWidget *browserentry;
   GtkWidget *pdfviewer;
+  GtkWidget *sequencer;
+  GtkWidget *midi_in;
   GtkWidget *texteditor;
   GtkWidget *denemopath;
   GtkWidget *temperament;
+  GtkWidget *strictshortcuts;
+  GtkWidget *resolution;
+  GtkWidget *overlays;
+  GtkWidget *continuous;
 };
 
 struct callbackdata1
@@ -113,11 +120,31 @@ set_preferences (struct callbackdata *cbdata)
   g_string_assign (prefs->denemopath,
                    gtk_entry_get_text (GTK_ENTRY (cbdata->denemopath)));
 
-#define ASSIGN(field) \
+#define ASSIGNTEXT(field) \
   g_string_assign (prefs->field,\
                    gtk_entry_get_text (GTK_ENTRY (cbdata->field)))
 
-  ASSIGN(temperament);
+
+  ASSIGNTEXT(sequencer);
+  ASSIGNTEXT(midi_in);
+
+#define ASSIGNBOOLEAN(field) \
+  prefs->field =\
+    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(cbdata->field));
+
+  ASSIGNTEXT(sequencer);
+  ASSIGNTEXT(midi_in);
+  ASSIGNTEXT(temperament);
+  ASSIGNBOOLEAN(strictshortcuts);
+  ASSIGNBOOLEAN(overlays);
+  ASSIGNBOOLEAN(continuous);
+
+#define ASSIGNINT(field) \
+   prefs->field =\
+    gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(cbdata->field));
+
+  ASSIGNINT(resolution);
+  ASSIGNINT(maxhistory);
 
   prefs->immediateplayback =
     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
@@ -161,12 +188,23 @@ preferences_change (GtkAction *action, gpointer param)
   GtkWidget *checkautosaveparts;
   GtkWidget *checkautosave;
   GtkWidget *autosaveentry;
+  GtkWidget *maxhistory;
   GtkWidget *pdfviewer;
+  GtkWidget *midi_in;
+  GtkWidget *sequencer;
   GtkWidget *texteditor;
   GtkWidget *denemopath;
   GtkWidget *checknotationpalette;
   GtkWidget *checkrhythmpalette;
   GtkWidget *checkarticulationpalette;
+  GtkWidget *strictshortcuts;
+  GtkWidget *resolution;
+  GtkWidget *overlays;
+  GtkWidget *continuous;
+
+
+
+
   GtkWidget *notebook;
   GtkWidget *hbox;
   GtkWidget *vbox;
@@ -175,6 +213,7 @@ preferences_change (GtkAction *action, gpointer param)
   GtkTreeIter iter;
   GtkTreeViewColumn *column;
   GtkCellRenderer *renderer;
+  GtkWidget *entrywidget;
   static struct callbackdata cbdata;
   static struct callbackdata1 cbdata1;
   g_assert (gui != NULL);
@@ -235,7 +274,6 @@ preferences_change (GtkAction *action, gpointer param)
 				Denemo.prefs.rhythm_palette);
   gtk_box_pack_start (GTK_BOX (main_vbox),checkrhythmpalette, FALSE, TRUE, 0);
 
-
   hbox = gtk_hbox_new (FALSE, 8);
   gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, TRUE, 0);
 
@@ -262,6 +300,21 @@ preferences_change (GtkAction *action, gpointer param)
 				Denemo.prefs.saveparts);
   gtk_box_pack_start (GTK_BOX (main_vbox), checkautosaveparts, FALSE, TRUE, 0);
 
+#define TEXTENTRY(thelabel, field) \
+  hbox = gtk_hbox_new (FALSE, 8);\
+  gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, TRUE, 0);\
+  label = gtk_label_new (_(thelabel));\
+  gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);\
+  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);\
+  entrywidget = gtk_entry_new ();\
+  gtk_entry_set_text (GTK_ENTRY (entrywidget), Denemo.prefs.field->str);\
+  gtk_box_pack_start (GTK_BOX (hbox), entrywidget, TRUE, TRUE, 0);\
+  cbdata.field = entrywidget;
+ 
+
+  TEXTENTRY("Sequencer Device", sequencer)
+  TEXTENTRY("Midi Input Device", midi_in)
+
 
   /*
    * Pitch Entry Parameters 
@@ -275,21 +328,38 @@ preferences_change (GtkAction *action, gpointer param)
   gtk_notebook_set_tab_label_text (GTK_NOTEBOOK (notebook), main_vbox,
 				   _("Pitch Entry"));
 
-  GtkWidget *entrywidget;
 
-#define TEXTENTRY(thelabel, field) \
-  hbox = gtk_hbox_new (FALSE, 8);\
-  gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, TRUE, 0);\
-  label = gtk_label_new (_(thelabel));\
-  gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);\
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);\
-  entrywidget = gtk_entry_new ();\
-  gtk_entry_set_text (GTK_ENTRY (entrywidget), Denemo.prefs.field->str);\
-  gtk_box_pack_start (GTK_BOX (hbox), entrywidget, TRUE, TRUE, 0);\
-  cbdata.field = entrywidget;
+
 
   TEXTENTRY("Temperament", temperament)
 
+#define BOOLEANENTRY(thelabel, field) \
+  field =\
+    gtk_check_button_new_with_label (thelabel); \
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (field),\
+				Denemo.prefs.field);\
+  gtk_box_pack_start (GTK_BOX (main_vbox), field, FALSE, TRUE, 0);
+
+  BOOLEANENTRY("Use Overlays", overlays);
+  BOOLEANENTRY("Continuous Entry", continuous);
+
+
+ /*
+   * Shortcut control 
+   */
+
+  main_vbox = gtk_vbox_new (FALSE, 1);
+  
+
+
+  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), main_vbox, NULL);
+  gtk_notebook_set_tab_label_text (GTK_NOTEBOOK (notebook), main_vbox,
+				   _("Shortcuts"));
+
+
+
+  //  TEXTENTRY("Strict", strictshortcuts)
+  BOOLEANENTRY("Strict Shortcuts", strictshortcuts);
 
   /*
    * External (Helper) Programs 
@@ -350,10 +420,10 @@ preferences_change (GtkAction *action, gpointer param)
   gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, FALSE, 0);
 
 
-
   /*
    * Plugins settings
    */
+  /*
   vbox = gtk_vbox_new (FALSE, 8);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, NULL);
@@ -421,6 +491,39 @@ preferences_change (GtkAction *action, gpointer param)
 						     COLUMN_PLUGIN, NULL);
 
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
+  */
+
+#define INTENTRY(thelabel, field) \
+  hbox = gtk_hbox_new (FALSE, 8);\
+  gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, TRUE, 0);\
+  label = gtk_label_new (thelabel);\
+  gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);\
+  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);\
+  field = gtk_spin_button_new_with_range (1, 50, 1.0);\
+  gtk_spin_button_set_value (GTK_SPIN_BUTTON (field), Denemo.prefs.field);\
+  gtk_box_pack_start (GTK_BOX (hbox), field, FALSE, FALSE, 0);
+
+#define INTENTRY_LIMITS(thelabel, field, min, max) \
+  hbox = gtk_hbox_new (FALSE, 8);\
+  gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, TRUE, 0);\
+  label = gtk_label_new (thelabel);\
+  gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);\
+  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);\
+  field = gtk_spin_button_new_with_range (min, max, 1.0);\
+  gtk_spin_button_set_value (GTK_SPIN_BUTTON (field), Denemo.prefs.field);\
+  gtk_box_pack_start (GTK_BOX (hbox), field, FALSE, FALSE, 0);
+
+ 
+  /*
+   * Excerpt Menu 
+   */
+
+  main_vbox = gtk_vbox_new (FALSE, 1);
+  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), main_vbox, NULL);
+  gtk_notebook_set_tab_label_text (GTK_NOTEBOOK (notebook), main_vbox,
+				   _("Excerpt"));
+
+  INTENTRY_LIMITS(_("resolution"), resolution, 72, 600);
 
   /*
    * Help settings
@@ -442,16 +545,29 @@ preferences_change (GtkAction *action, gpointer param)
   gtk_entry_set_text (GTK_ENTRY (browserentry), Denemo.prefs.browser->str);
   gtk_box_pack_start (GTK_BOX (hbox), browserentry, TRUE, TRUE, 0);
 
+  INTENTRY(_("Max recent files"), maxhistory);
+
   /* Set up the callback data */
   cbdata.prefs = &Denemo.prefs;
   cbdata.lilypathentry = lilypathentry;
   cbdata.checkimmediateplayback = checkimmediateplayback;
+
+#define SETCALLBACKDATA(field) \
+  cbdata.field = field;
+
+
+  SETCALLBACKDATA(strictshortcuts);
+  SETCALLBACKDATA(resolution);  
+  SETCALLBACKDATA(overlays);
+  SETCALLBACKDATA(continuous);
+
   cbdata.checkautosaveparts = checkautosaveparts;
   cbdata.checkarticulationpalette = checkarticulationpalette;
   cbdata.checknotationpalette = checknotationpalette;
   cbdata.checkrhythmpalette = checkrhythmpalette;
   cbdata.checkautosave = checkautosave;
   cbdata.autosaveentry = autosaveentry;
+  cbdata.maxhistory = maxhistory;
   cbdata.browserentry = browserentry;
   cbdata.pdfviewer = pdfviewer;
   cbdata.texteditor = texteditor;
@@ -460,6 +576,7 @@ preferences_change (GtkAction *action, gpointer param)
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
   gtk_entry_set_activates_default (GTK_ENTRY (lilypathentry), TRUE);
   gtk_entry_set_activates_default (GTK_ENTRY (autosaveentry), TRUE);
+  gtk_entry_set_activates_default (GTK_ENTRY (maxhistory), TRUE);
   gtk_entry_set_activates_default (GTK_ENTRY (browserentry), TRUE);
   gtk_entry_set_activates_default (GTK_ENTRY (pdfviewer), TRUE);
   gtk_entry_set_activates_default (GTK_ENTRY (texteditor), TRUE);
