@@ -78,11 +78,19 @@ initprefs ()
   ret->browser = g_string_new ("firefox");
   ret->csoundorcfile = g_string_new ("");
   ret->pdfviewer = g_string_new ("xpdf");
+  ret->sequencer = g_string_new ("/dev/sequencer");
+  ret->midi_in = g_string_new ("/dev/midi");
+
+
   ret->imageviewer = g_string_new ("eog");
   ret->texteditor = g_string_new ("xedit");
   ret->denemopath = g_string_new (g_get_home_dir());
   ret->lilyversion = g_string_new (LILYPOND_VERSION);
   ret->temperament = g_string_new("Equal");
+  ret->strictshortcuts = FALSE;
+  ret->resolution = 300;
+  ret->overlays = FALSE;
+  ret->continuous = TRUE;
 
   ret->immediateplayback = TRUE;
   ret->saveparts = FALSE;
@@ -92,6 +100,7 @@ initprefs ()
   ret->createclones = FALSE;
   ret->autosave = TRUE;
   ret->autosave_timeout = 5;
+  ret->maxhistory = 20;
   ret->notation_palette = TRUE;
   ret->articulation_palette = FALSE;
   ret->rhythm_palette = TRUE;
@@ -140,7 +149,7 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
 	    {
 	      prefs->lilypath = 
 		g_string_assign (prefs->lilypath, (gchar *) tmp);
-	      g_print ("Lilypond Path %s\n", tmp);
+	      //g_print ("Lilypond Path %s\n", tmp);
 	      xmlFree (tmp);
 	    }
 	}
@@ -151,7 +160,7 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
 	    {
 	      prefs->midiplayer =
 		g_string_assign (prefs->midiplayer, (gchar *) tmp);
-	      g_print ("midiplayer %s\n", tmp);
+	      //g_print ("midiplayer %s\n", tmp);
 	      xmlFree (tmp);
 	    }
 	}
@@ -162,7 +171,7 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
 	    {
 	      prefs->audioplayer =
 		g_string_assign (prefs->audioplayer, (gchar *) tmp);
-	      g_print ("Audioplayer %s\n", tmp);
+	      //g_print ("Audioplayer %s\n", tmp);
 	      xmlFree (tmp);
 	    }
 	}
@@ -172,7 +181,7 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
 	  if(tmp)
 	    {
 	      prefs->browser = g_string_assign (prefs->browser, (gchar *) tmp);
-	      g_print ("Help Browser %s\n", tmp);
+	      //g_print ("Help Browser %s\n", tmp);
 	      xmlFree (tmp);
 	    }
 	}
@@ -184,10 +193,23 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
 	    {
 	      prefs->autosave_timeout = atoi ((gchar *) tmp);
 	      if(prefs->autosave_timeout <1) prefs->autosave_timeout = 1;
-	      g_print ("Autosave Timeout %s\n", tmp);
+	      //g_print ("Autosave Timeout %s\n", tmp);
 	      xmlFree (tmp);
 	    }
 	}
+      else if (0 ==
+	       xmlStrcmp (cur->name, (const xmlChar *) "maxhistory"))
+	{
+	  xmlChar *tmp = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
+	  if(tmp)
+	    {
+	      prefs->maxhistory = atoi ((gchar *) tmp);
+	      if(prefs->maxhistory <1) prefs->maxhistory = 1;
+	      xmlFree (tmp);
+	    }
+	}
+
+
       else if (0 == xmlStrcmp (cur->name, (const xmlChar *) "csoundcommand"))
 	{
 	  xmlChar *tmp = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
@@ -195,7 +217,7 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
 	    {
 	      prefs->csoundcommand =
 		g_string_assign (prefs->csoundcommand, (gchar *) tmp);
-	      g_print ("Csound Command %s\n", tmp);
+	      //g_print ("Csound Command %s\n", tmp);
 	      xmlFree (tmp);
 	    }
 	}
@@ -206,7 +228,7 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
 	    {
 	      prefs->csoundorcfile =
 		g_string_assign (prefs->csoundorcfile, (gchar *) tmp);
-	      g_print ("Csound Orchestrafile %s\n", tmp);
+	      //g_print ("Csound Orchestrafile %s\n", tmp);
 	      xmlFree (tmp);
 	    }
 
@@ -287,6 +309,9 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
 	}
 
       READXMLENTRY(temperament)
+      READXMLENTRY(midi_in)
+      READXMLENTRY(sequencer)
+
       else if (0 == xmlStrcmp (cur->name, (const xmlChar *) "createclones"))
 	{
 	  xmlChar *tmp = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
@@ -308,6 +333,25 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
 	    }
 	}
 
+#define READINTXMLENTRY(field) \
+      else if (0 ==\
+	       xmlStrcmp (cur->name, (const xmlChar *) #field))\
+	{\
+	  xmlChar *tmp = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);\
+	  if(tmp)\
+	    {\
+	      prefs->field = atoi ((gchar *) tmp);\
+	      xmlFree (tmp);\
+	    }\
+	}\
+
+
+
+      READINTXMLENTRY(strictshortcuts)
+      READINTXMLENTRY(resolution)
+      READINTXMLENTRY(overlays)
+      READINTXMLENTRY(continuous)
+
       else if (0 == xmlStrcmp (cur->name, (const xmlChar *) "lilystyleentry"))
 	{
 	  xmlChar *tmp = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
@@ -363,7 +407,7 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
 	  if(tmp)
 	    {
 	      prefs->saveparts = atoi((gchar *) tmp);
-	      g_print ("Autosave Parts %s\n", tmp);
+	      //g_print ("Autosave Parts %s\n", tmp);
 	      xmlFree(tmp);	
 	    }
 	}
@@ -380,7 +424,7 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
 static void
 writeHistoryEntry (gpointer data, gpointer user_data)
 {
-  g_print ("filename %s\n", (gchar *) data);
+  //g_print ("filename %s\n", (gchar *) data);
   xmlNewTextChild ((xmlNodePtr) user_data, NULL, (xmlChar *) "file",
 		   (xmlChar *) data);
 }
@@ -563,6 +607,8 @@ writeXMLPrefs (DenemoPrefs * prefs)
 		 (xmlChar *) prefs->field->str)
 
   WRITEXMLENTRY(temperament);
+  WRITEXMLENTRY(midi_in);
+  WRITEXMLENTRY(sequencer);
 
   if (prefs->lilyversion)
     xmlNewChild (child, NULL, (xmlChar *) "lilyversion",
@@ -570,11 +616,23 @@ writeXMLPrefs (DenemoPrefs * prefs)
   newXMLIntChild (child, (xmlChar *) "autosave", prefs->autosave);
   newXMLIntChild (child, (xmlChar *) "autosavetimeout",
 		  prefs->autosave_timeout);
+  newXMLIntChild (child, (xmlChar *) "maxhistory",
+		  prefs->maxhistory);
   newXMLIntChild (child, (xmlChar *) "saveparts", prefs->saveparts);
   newXMLIntChild (child, (xmlChar *) "createclones", prefs->createclones);
   newXMLIntChild (child, (xmlChar *) "lilystyleentry", prefs->lilyentrystyle);
   newXMLIntChild (child, (xmlChar *) "immediateplayback",
 		  prefs->immediateplayback);
+
+#define WRITEINTXMLENTRY(field) \
+  newXMLIntChild (child, (xmlChar *) #field,\
+		  prefs->field);
+
+  WRITEINTXMLENTRY(strictshortcuts);
+  WRITEINTXMLENTRY(resolution);
+  WRITEINTXMLENTRY(overlays);
+  WRITEINTXMLENTRY(continuous);
+
   newXMLIntChild (child, (xmlChar *) "rtcs", prefs->rtcs);
   newXMLIntChild (child, (xmlChar *) "notationpalette",
 		  prefs->notation_palette);
@@ -700,7 +758,7 @@ void storeWindowState (void)
    GKeyFile *keyfile;
    gchar *contents;
    gchar *filename;
-   gtk_window_get_size (Denemo.window, &(Denemo.width), &(Denemo.height));
+   gtk_window_get_size ( GTK_WINDOW (Denemo.window), &(Denemo.width), &(Denemo.height));
    keyfile = g_key_file_new ();
    g_key_file_set_integer (keyfile, "State", "width", Denemo.width);
    g_key_file_set_integer (keyfile, "State", "height", Denemo.height);
@@ -724,41 +782,43 @@ void loadWindowState (void)
     gchar *filename;
     GKeyFile *keyfile;
     gint w,h;
-    gboolean maximized;
+    gboolean maximized=FALSE;
     GError *err=NULL;
-
     filename = g_build_filename (locatedotdenemo (), "state.ini", NULL);
     keyfile = g_key_file_new ();
     if (g_key_file_load_from_file (keyfile, filename, G_KEY_FILE_NONE, NULL) == FALSE) {
         g_free (filename);
         w = INITIAL_WIDTH;
         h = INITIAL_HEIGHT;
+	maximized = FALSE;
     } else {
         g_free (filename);
         w = g_key_file_get_integer (keyfile, "State", "width", &err);
         if (err != NULL) {
-            w = 0;
+            w = INITIAL_WIDTH;
             g_error_free (err);
             err = NULL;
         }
         h = g_key_file_get_integer (keyfile, "State", "height", &err);
         if (err != NULL) {
-            h = 0;
+            h = INITIAL_HEIGHT;
             g_error_free (err);
             err = NULL;
         }
         maximized = g_key_file_get_boolean (keyfile, "State", "maximized", &err);
         if (err != NULL) {
-            g_error_free (err);
-            err = NULL;
+	  maximized = FALSE;
+	  g_error_free (err);
+	  err = NULL;
         }
+
         g_key_file_free (keyfile);
-        if (w > 0 && h > 0 && maximized == FALSE) {
-                gtk_window_set_default_size (GTK_WINDOW (Denemo.window), w, h);
-                Denemo.width = w;
-                Denemo.height = h;
-        } else if (Denemo.maximized != FALSE) {
-                gtk_window_maximize (GTK_WINDOW (Denemo.window));
-        }
+	Denemo.width = (w<=0?INITIAL_WIDTH:w);
+	Denemo.height = (h<=0?INITIAL_HEIGHT:h);
+	gtk_window_set_default_size (GTK_WINDOW (Denemo.window), Denemo.width, Denemo.height);
+        if ((Denemo.maximized=maximized)) {
+	  gtk_window_maximize (GTK_WINDOW (Denemo.window));
+        } else
+	  gtk_window_unmaximize (GTK_WINDOW (Denemo.window));
     }
 }
