@@ -4,19 +4,19 @@
 #include <denemo/denemo.h> 
 
 void
-printall_cb(GtkAction *action, DenemoGUI *gui);
+printall_cb(GtkAction *action, gpointer param);
 void
-printpreview_cb(GtkAction *action, DenemoGUI *gui);
+printpreview_cb(GtkAction *action, gpointer param);
 void
-printexcerptpreview_cb(GtkAction *action, DenemoGUI *gui);
+printexcerptpreview_cb(GtkAction *action, gpointer param);
 void
-printpart_cb(GtkAction *action, DenemoGUI *gui);
+printpart_cb(GtkAction *action, gpointer param);
 gchar *
 get_printfile_pathbasename(void);
 void
 run_lilypond_and_viewer(gchar *basename, DenemoGUI *gui);
-void export_pdf_action (GtkAction *action, DenemoGUI *gui);
-void export_pdf (const gchar* filename, DenemoGUI *gui);
+void export_pdf_action (GtkAction *action, gpointer param);
+
 
 #endif /*PRINT_H*/	
 /* exportpdf.h
