@@ -177,7 +177,7 @@ next_movement (GtkAction *action, gpointer param)
   update_vscrollbar (gui);
   gtk_signal_emit_by_name (GTK_OBJECT (gui->hadjustment), "changed");
   gtk_signal_emit_by_name (GTK_OBJECT (gui->vadjustment), "changed");
-  gtk_widget_queue_draw (gui->scorearea);
+  gtk_widget_draw (gui->scorearea, NULL);//KLUDGE FIXME see staffup/down
 }
 
 /**
@@ -208,7 +208,7 @@ prev_movement (GtkAction *action, gpointer param)
   gtk_widget_queue_draw (gui->scorearea);
   gtk_signal_emit_by_name (GTK_OBJECT (gui->hadjustment), "changed");
   gtk_signal_emit_by_name (GTK_OBJECT (gui->vadjustment), "changed");
-
+ gtk_widget_draw (gui->scorearea, NULL);//KLUDGE FIXME see staffup/down
 }
 /**
  * Initialise scoreinfo structure 
@@ -240,7 +240,6 @@ init_score (DenemoScore * si, DenemoGUI *gui)
   si->markmeasurenum = 0;
   si->markcursor_x = 0;
   si->maxkeywidth = 0;
-  si->is_grace_mode = FALSE;
   si->has_figures = FALSE;
   si->has_fakechords = FALSE;
   /*playback purposes */
@@ -271,6 +270,7 @@ init_score (DenemoScore * si, DenemoGUI *gui)
   si->headerinfo.extra = g_string_new ("breakbefore = ##f");
   HEADER(lilypond_before);
   HEADER(lilypond_after);
+  HEADER(layout);
 #undef HEADER
   if(gui->filename==NULL) {
     gui->filename = g_string_new ("");
@@ -338,6 +338,7 @@ free_score (DenemoGUI * gui)
   HEADER(tagline);
   HEADER(lilypond_before);
   HEADER(lilypond_after);
+  HEADER(layout);
   HEADER(extra);
 #undef HEADER
 
