@@ -14,12 +14,12 @@ void new_score(DenemoGUI * gui);
 void init_score (DenemoScore *si, DenemoGUI * gui);
 
 void free_score (DenemoGUI *gui);
-void next_movement (GtkAction * action, DenemoGUI * gui);
-void prev_movement (GtkAction * action, DenemoGUI * gui);
-void insert_movement_before (GtkAction * action, DenemoGUI *gui);
-void insert_movement_after (GtkAction * action, DenemoGUI *gui);
-void delete_movement (GtkAction * action, DenemoGUI * gui);
-void movement_props_dialog (GtkAction * action, DenemoGUI * gui);
+void next_movement (GtkAction *action, gpointer param);
+void prev_movement (GtkAction *action, gpointer param);
+void insert_movement_before (GtkAction *action, gpointer param);
+void insert_movement_after (GtkAction *action, gpointer param);
+void delete_movement (GtkAction *action, gpointer param);
+void movement_props_dialog (GtkAction *action, gpointer param);
 gboolean
 goto_movement_staff_obj (DenemoGUI * gui, gint movementnum, gint staffnum, gint measurenum, gint objnum);
 #endif
