@@ -461,6 +461,102 @@ unset_mark (DenemoGUI * gui)
   calcmarkboundaries (si);
 }
 
+
+gboolean in_selection(DenemoScore *si) {
+  if(si->currentstaffnum >= si->firststaffmarked &&
+     si->currentstaffnum <= si->laststaffmarked) {
+    if(si->currentmeasurenum == si->firstmeasuremarked) {
+      if (si->currentmeasurenum == si->lastmeasuremarked) {
+	if((si->cursor_x >= si->firstobjmarked) &&
+	   (si->cursor_x <= si->lastobjmarked))
+	  return TRUE;
+	else return FALSE;
+      }
+      if (si->currentmeasurenum < si->lastmeasuremarked){
+	if(si->cursor_x >= si->firstobjmarked)
+	  return TRUE;
+	return FALSE;
+      }
+    }
+    if(si->currentmeasurenum > si->firstmeasuremarked) {
+      if (si->currentmeasurenum == si->lastmeasuremarked) {
+	if((si->cursor_x <= si->lastobjmarked))
+	  return TRUE;
+	else return FALSE;
+      }
+      if (si->currentmeasurenum < si->lastmeasuremarked)
+	return TRUE;
+    }
+  }
+  return FALSE;  
+}
+
+/* save/restore selection */
+static      gint firststaff;
+static      gint laststaff;
+static      gint firstobj;
+static      gint lastobj;
+static      gint firstmeasure;
+static      gint lastmeasure;
+
+void save_selection(DenemoScore *si) {
+  firststaff = si->firststaffmarked;
+  laststaff = si->laststaffmarked;
+  firstobj = si->firstobjmarked;
+  lastobj = si->lastobjmarked;
+  firstmeasure =si->firstmeasuremarked;
+  lastmeasure =si->lastmeasuremarked;
+}
+void restore_selection(DenemoScore *si) {
+  si->firststaffmarked = firststaff;
+  si->laststaffmarked = laststaff;
+  si->firstobjmarked  = firstobj;
+  si->lastobjmarked = lastobj;
+  si->firstmeasuremarked = firstmeasure;
+  si->lastmeasuremarked = lastmeasure;
+}
+
+
+void
+goto_nearestnote (GtkAction *action, gpointer param) {
+#if 0
+  if(gui->si->currentobject) {
+    objnode *thenotenode =  nearestnote (gui->si->currentobject->data, gui->si->cursor_y);
+    if(thenotenode) {
+      note *thenote =  *((note *) thenotenode->data);
+      //thenote->mid_c_offset!!!!!!!!!!!
+    }
+
+  }
+#endif
+}
+
+/**
+ * goto_mark
+ * goto the current mark without changing the selection
+ *
+ * 
+ */
+void
+goto_mark (GtkAction *action, gpointer param)
+{
+  DenemoScore *si = Denemo.gui->si;
+  if(!action)
+    ((DenemoScriptParam *)param)->status = si->markstaffnum;
+  if(si->markstaffnum){
+    save_selection(si);
+    set_currentmeasurenum (Denemo.gui, si->markmeasurenum);
+    set_currentstaffnum (Denemo.gui,si->markstaffnum);
+    while(si->cursor_x < si->markcursor_x)
+      cursorright(param);
+    restore_selection(si);
+    displayhelper(Denemo.gui);
+  } 
+}
+
+
+
+
 /**
  *  copywrapper
  *  Wrapper function for the copy command
