@@ -162,7 +162,6 @@ insert_staff (DenemoScore * si, DenemoStaff * thestaffstruct,
       find_leftmost_staffcontext (thestaffstruct, si);
     }
 
-
 }
 
 
@@ -292,7 +291,8 @@ newstaff (DenemoGUI * gui, enum newstaffcallbackaction action,
 		      numstaffs + 1);
   set_lily_name (thestaffstruct->denemo_name, thestaffstruct->lily_name);
   thestaffstruct->midi_instrument = g_string_new ("acoustic grand");
-
+  thestaffstruct->leftmost_time1context = thestaffstruct->stime1;
+      thestaffstruct->leftmost_time2context = thestaffstruct->stime2;
   /* In what position should the scrollbar be added?  */
   switch (action)
     {
@@ -433,10 +433,10 @@ deletestaff (DenemoGUI * gui, gboolean interactive)
 
   //FIXME none of this works to get the current measure stem direction correct
       setcurrents (si);
-      find_xes_in_all_measures (si);
-      beamsandstemdirswholestaff ((DenemoStaff *) si->currentstaff->data);
-      si->markstaffnum = 0;
-      displayhelper (gui);
+      //     find_xes_in_all_measures (si);
+      //   beamsandstemdirswholestaff ((DenemoStaff *) si->currentstaff->data);
+      //   si->markstaffnum = 0;
+      //  displayhelper (gui);
   score_status(gui,TRUE);
   if(give_info)
     infodialog ("The staff deleted had a start/end context; if you still have the staff with the matching end/start context\n then you should remove it (or its context) now.\nSee Staff->properties->context\nYou will not be able to print with miss-matched contexts.");
@@ -544,7 +544,7 @@ newstaffinitial (GtkAction *action, gpointer param)
 {
   DenemoGUI *gui = Denemo.gui;
   while(gui->si->currentstaff && gui->si->currentstaff->prev)
-    staffup(gui);
+    staffup(param);
   newstaffbefore (action, NULL);
 }
 
@@ -569,7 +569,7 @@ newstaffbefore (GtkAction *action, gpointer param)
   set_bottom_staff (gui);
   move_viewport_down (gui);
 
-  staffup (gui);
+  staffup (param);
   displayhelper (gui);
 
 }
@@ -590,7 +590,7 @@ dnm_newstaffafter (GtkAction *action, gpointer param)
   newstaff (gui, AFTER, DENEMO_NONE);
   set_bottom_staff (gui);
   update_vscrollbar (gui);
-  staffdown (gui);
+  staffdown (param);
   displayhelper (gui);
 }
 
@@ -605,7 +605,7 @@ newstafflast (GtkAction *action, gpointer param)
 {
   DenemoGUI *gui = Denemo.gui;
   while(gui->si->currentstaff && gui->si->currentstaff->next)
-    staffdown(gui);
+    staffdown(param);
   dnm_newstaffafter(action, param);
 }
 
@@ -622,7 +622,7 @@ dnm_newstaffvoice (GtkAction *action, gpointer param)
   newstaff (gui, NEWVOICE, DENEMO_NONE);
   set_bottom_staff (gui);
   update_vscrollbar (gui);
-  setcurrents(gui->si);//staffdown (gui);
+  setcurrents(gui->si);
   displayhelper (gui);
 }
 
