@@ -62,11 +62,17 @@ void	  highlight_rhythm(RhythmPattern *r);
 void	  unhighlight_rhythm(RhythmPattern *r);
 void	  highlight_duration(DenemoGUI *gui, gint dur);
 void	  highlight_rest(DenemoGUI *gui, gint dur);
-void      write_status(DenemoGUI *gui);
 
 
+gchar *getSchemeText(void); 
+void deleteSchemeText(void);
+void appendSchemeText(gchar *text);
+void executeScript(void);
+gint getNumCharsSchemeText(void);
 void
 activate_script (GtkAction *action, gpointer param);
 void 
+create_scheme_function_for_script(gchar *name);
+void 
 inner_main(void*closure, int argc, char **argv);
 #endif
